@@ -1,19 +1,29 @@
-# Author: Jean-Remi King <jeanremi.king@gmail.com>
-#
+# Authors: The MNE-Python contributors.
 # License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
+
+import logging
 
 import numpy as np
+from sklearn.base import BaseEstimator, MetaEstimatorMixin, clone
+from sklearn.metrics import check_scoring
+from sklearn.preprocessing import LabelEncoder
+from sklearn.utils.validation import check_is_fitted
 
-from .mixin import TransformerMixin
-from .base import BaseEstimator, _check_estimator
-from ..fixes import _get_check_scoring
 from ..parallel import parallel_func
-from ..utils import (_validate_type, array_split_idx, ProgressBar,
-                     verbose, fill_doc)
+from ..utils import (
+    ProgressBar,
+    _parse_verbose,
+    _verbose_safe_false,
+    array_split_idx,
+    fill_doc,
+)
+from .base import _check_estimator
+from .transformer import MNETransformerMixin
 
 
 @fill_doc
-class SlidingEstimator(BaseEstimator, TransformerMixin):
+class SlidingEstimator(MetaEstimatorMixin, MNETransformerMixin, BaseEstimator):
     """Search Light.
 
     Fit, predict and score a series of models to each subset of the dataset
@@ -25,6 +35,8 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
     %(base_estimator)s
     %(scoring)s
     %(n_jobs)s
+    %(position)s
+    %(allow_2d)s
     %(verbose)s
 
     Attributes
@@ -33,25 +45,48 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         List of fitted scikit-learn estimators (one per task).
     """
 
-    def __init__(self, base_estimator, scoring=None, n_jobs=1,
-                 verbose=None):  # noqa: D102
-        _check_estimator(base_estimator)
-        self._estimator_type = getattr(base_estimator, "_estimator_type", None)
+    def __init__(
+        self,
+        base_estimator,
+        scoring=None,
+        n_jobs=None,
+        *,
+        position=0,
+        allow_2d=False,
+        verbose=None,
+    ):
         self.base_estimator = base_estimator
         self.n_jobs = n_jobs
         self.scoring = scoring
+        self.position = position
+        self.allow_2d = allow_2d
         self.verbose = verbose
 
-        _validate_type(self.n_jobs, 'int', 'n_jobs')
+    @property
+    def _estimator_type(self):
+        return getattr(self.base_estimator, "_estimator_type", None)
+
+    def __sklearn_tags__(self):
+        """Get sklearn tags."""
+        from sklearn.utils import get_tags
+
+        tags = super().__sklearn_tags__()
+        sub_tags = get_tags(self.base_estimator)
+        tags.estimator_type = sub_tags.estimator_type
+        for kind in ("classifier", "regressor", "transformer"):
+            if tags.estimator_type == kind:
+                attr = f"{kind}_tags"
+                setattr(tags, attr, getattr(sub_tags, attr))
+                break
+        return tags
 
     def __repr__(self):  # noqa: D105
-        repr_str = '<' + super(SlidingEstimator, self).__repr__()
-        if hasattr(self, 'estimators_'):
+        repr_str = "<" + super().__repr__()
+        if hasattr(self, "estimators_"):
             repr_str = repr_str[:-1]
-            repr_str += ', fitted with %i estimators' % len(self.estimators_)
-        return repr_str + '>'
+            repr_str += f", fitted with {len(self.estimators_)} estimators"
+        return repr_str + ">"
 
-    @verbose  # to use class value
     def fit(self, X, y, **fit_params):
         """Fit a series of independent estimators to the dataset.
 
@@ -72,19 +107,24 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         self : object
             Return self.
         """
-        self._check_Xy(X, y)
+        _check_estimator(self.base_estimator)
+        X, _ = self._check_Xy(X, y, fit=True)
+        parallel, p_func, n_jobs = parallel_func(
+            _sl_fit,
+            self.n_jobs,
+            max_jobs=X.shape[-1],
+            verbose=_verbose_safe_false(),
+        )
         self.estimators_ = list()
-        self.fit_params = fit_params
+        self.fit_params_ = fit_params
+
         # For fitting, the parallelization is across estimators.
-        parallel, p_func, n_jobs = parallel_func(_sl_fit, self.n_jobs,
-                                                 verbose=False)
-        n_jobs = min(n_jobs, X.shape[-1])
-        mesg = 'Fitting %s' % (self.__class__.__name__,)
-        with ProgressBar(X.shape[-1], mesg=mesg) as pb:
+        context = _create_progressbar_context(self, X, "Fitting")
+        with context as pb:
             estimators = parallel(
-                p_func(self.base_estimator, split, y, pb.subset(pb_idx),
-                       **fit_params)
-                for pb_idx, split in array_split_idx(X, n_jobs, axis=-1))
+                p_func(self.base_estimator, split, y, pb.subset(pb_idx), **fit_params)
+                for pb_idx, split in array_split_idx(X, n_jobs, axis=-1)
+            )
 
         # Each parallel job can have a different number of training estimators
         # We can't directly concatenate them because of sklearn's Bagging API
@@ -120,28 +160,38 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         """  # noqa: E501
         return self.fit(X, y, **fit_params).transform(X)
 
-    @verbose  # to use the class value
     def _transform(self, X, method):
         """Aux. function to make parallel predictions/transformation."""
-        self._check_Xy(X)
+        X, is_nd = self._check_Xy(X)
+        orig_method = method
+        check_is_fitted(self)
         method = _check_method(self.base_estimator, method)
         if X.shape[-1] != len(self.estimators_):
-            raise ValueError('The number of estimators does not match '
-                             'X.shape[-1]')
+            raise ValueError("The number of estimators does not match X.shape[-1]")
         # For predictions/transforms the parallelization is across the data and
         # not across the estimators to avoid memory load.
-        mesg = 'Transforming %s' % (self.__class__.__name__,)
         parallel, p_func, n_jobs = parallel_func(
-            _sl_transform, self.n_jobs, verbose=False)
-        n_jobs = min(n_jobs, X.shape[-1])
+            _sl_transform,
+            self.n_jobs,
+            max_jobs=X.shape[-1],
+            verbose=_verbose_safe_false(),
+        )
+
         X_splits = np.array_split(X, n_jobs, axis=-1)
         idx, est_splits = zip(*array_split_idx(self.estimators_, n_jobs))
-        with ProgressBar(X.shape[-1], mesg=mesg) as pb:
-            y_pred = parallel(p_func(est, x, method, pb.subset(pb_idx))
-                              for pb_idx, est, x in zip(
-                                  idx, est_splits, X_splits))
+
+        context = _create_progressbar_context(self, X, "Transforming")
+        with context as pb:
+            y_pred = parallel(
+                p_func(est, x, method, pb.subset(pb_idx))
+                for pb_idx, est, x in zip(idx, est_splits, X_splits)
+            )
 
         y_pred = np.concatenate(y_pred, axis=1)
+        if orig_method == "transform":
+            y_pred = y_pred.astype(X.dtype)
+        if orig_method == "predict_proba" and not is_nd:
+            y_pred = y_pred[:, 0, :]
         return y_pred
 
     def transform(self, X):
@@ -164,7 +214,7 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         Xt : array, shape (n_samples, n_estimators)
             The transformed values generated by each estimator.
         """  # noqa: E501
-        return self._transform(X, 'transform')
+        return self._transform(X, "transform")
 
     def predict(self, X):
         """Predict each data slice/task with a series of independent estimators.
@@ -186,7 +236,7 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         y_pred : array, shape (n_samples, n_estimators) | (n_samples, n_tasks, n_targets)
             Predicted values for each estimator/data slice.
         """  # noqa: E501
-        return self._transform(X, 'predict')
+        return self._transform(X, "predict")
 
     def predict_proba(self, X):
         """Predict each data slice with a series of independent estimators.
@@ -208,7 +258,7 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         y_pred : array, shape (n_samples, n_tasks, n_classes)
             Predicted probabilities for each estimator/data slice/task.
         """  # noqa: E501
-        return self._transform(X, 'predict_proba')
+        return self._transform(X, "predict_proba")
 
     def decision_function(self, X):
         """Estimate distances of each data slice to the hyperplanes.
@@ -231,15 +281,23 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         -----
         This requires base_estimator to have a ``decision_function`` method.
         """  # noqa: E501
-        return self._transform(X, 'decision_function')
+        return self._transform(X, "decision_function")
 
-    def _check_Xy(self, X, y=None):
+    def _check_Xy(self, X, y=None, fit=False):
         """Aux. function to check input data."""
-        if y is not None:
-            if len(X) != len(y) or len(y) < 1:
-                raise ValueError('X and y must have the same length.')
-        if X.ndim < 3:
-            raise ValueError('X must have at least 3 dimensions.')
+        # Once we require sklearn 1.1+ we should do something like:
+        X = self._check_data(X, y=y, atleast_3d=False, fit=fit)
+        is_nd = X.ndim >= 3
+        if not is_nd:
+            err = None
+            if not self.allow_2d:
+                err = 3
+            elif X.ndim < 2:
+                err = 2
+            if err:
+                raise ValueError(f"X must have at least {err} dimensions.")
+            X = X[..., np.newaxis]
+        return X, is_nd
 
     def score(self, X, y):
         """Score each estimator on each task.
@@ -264,34 +322,37 @@ class SlidingEstimator(BaseEstimator, TransformerMixin):
         score : array, shape (n_samples, n_estimators)
             Score for each estimator/task.
         """  # noqa: E501
-        check_scoring = _get_check_scoring()
-
-        self._check_Xy(X)
+        X, _ = self._check_Xy(X, y)
         if X.shape[-1] != len(self.estimators_):
-            raise ValueError('The number of estimators does not match '
-                             'X.shape[-1]')
+            raise ValueError("The number of estimators does not match X.shape[-1]")
 
         scoring = check_scoring(self.base_estimator, self.scoring)
         y = _fix_auc(scoring, y)
 
         # For predictions/transforms the parallelization is across the data and
         # not across the estimators to avoid memory load.
-        parallel, p_func, n_jobs = parallel_func(_sl_score, self.n_jobs)
-        n_jobs = min(n_jobs, X.shape[-1])
+        parallel, p_func, n_jobs = parallel_func(
+            _sl_score,
+            self.n_jobs,
+            max_jobs=X.shape[-1],
+            verbose=_verbose_safe_false(),
+        )
         X_splits = np.array_split(X, n_jobs, axis=-1)
         est_splits = np.array_split(self.estimators_, n_jobs)
-        score = parallel(p_func(est, scoring, x, y)
-                         for (est, x) in zip(est_splits, X_splits))
+        score = parallel(
+            p_func(est, scoring, x, y) for (est, x) in zip(est_splits, X_splits)
+        )
 
         score = np.concatenate(score, axis=0)
         return score
 
     @property
     def classes_(self):
-        if not hasattr(self.estimators_[0], 'classes_'):
-            raise AttributeError('classes_ attribute available only if '
-                                 'base_estimator has it, and estimator %s does'
-                                 ' not' % (self.estimators_[0],))
+        if not hasattr(self.estimators_[0], "classes_"):
+            raise AttributeError(
+                "classes_ attribute available only if base_estimator has it, and "
+                f"estimator {self.estimators_[0]} does not"
+            )
         return self.estimators_[0].classes_
 
 
@@ -309,6 +370,8 @@ def _sl_fit(estimator, X, y, pb, **fit_params):
         X.shape = (n_samples, n_features_1, n_features_2, n_estimators)
     y : array, shape (n_sample, )
         The target values.
+    pb : instance of ProgressBar
+        The progress bar to update.
     fit_params : dict | None
         Parameters to pass to the fit method of the estimator.
 
@@ -317,12 +380,12 @@ def _sl_fit(estimator, X, y, pb, **fit_params):
     estimators_ : list of estimators
         The fitted estimators.
     """
-    from sklearn.base import clone
     estimators_ = list()
     for ii in range(X.shape[-1]):
         est = clone(estimator)
         est.fit(X[..., ii], y, **fit_params)
         estimators_.append(est)
+
         pb.update(ii + 1)
     return estimators_
 
@@ -341,6 +404,8 @@ def _sl_transform(estimators, X, method, pb):
         X.shape = (n_samples, n_features_1, n_features_2, n_estimators)
     method : str
         The estimator method to use (e.g. 'predict', 'transform').
+    pb : instance of ProgressBar
+        The progress bar to update.
 
     Returns
     -------
@@ -354,6 +419,7 @@ def _sl_transform(estimators, X, method, pb):
         if ii == 0:
             y_pred = _sl_init_pred(_y_pred, X)
         y_pred[:, ii, ...] = _y_pred
+
         pb.update(ii + 1)
     return y_pred
 
@@ -402,10 +468,10 @@ def _check_method(estimator, method):
     If method == 'transform'  and estimator does not have 'transform', use
     'predict' instead.
     """
-    if method == 'transform' and not hasattr(estimator, 'transform'):
-        method = 'predict'
+    if method == "transform" and not hasattr(estimator, "transform"):
+        method = "predict"
     if not hasattr(estimator, method):
-        ValueError('base_estimator does not have `%s` method.' % method)
+        ValueError(f"base_estimator does not have `{method}` method.")
     return method
 
 
@@ -421,32 +487,46 @@ class GeneralizingEstimator(SlidingEstimator):
     %(base_estimator)s
     %(scoring)s
     %(n_jobs)s
+    %(position)s
+    %(allow_2d)s
     %(verbose)s
     """
 
     def __repr__(self):  # noqa: D105
-        repr_str = super(GeneralizingEstimator, self).__repr__()
-        if hasattr(self, 'estimators_'):
+        repr_str = super().__repr__()
+        if hasattr(self, "estimators_"):
             repr_str = repr_str[:-1]
-            repr_str += ', fitted with %i estimators>' % len(self.estimators_)
+            repr_str += f", fitted with {len(self.estimators_)} estimators>"
         return repr_str
 
-    @verbose  # use class value
     def _transform(self, X, method):
         """Aux. function to make parallel predictions/transformation."""
-        self._check_Xy(X)
+        X, is_nd = self._check_Xy(X)
+        check_is_fitted(self)
+        orig_method = method
         method = _check_method(self.base_estimator, method)
-        mesg = 'Transforming %s' % (self.__class__.__name__,)
+
         parallel, p_func, n_jobs = parallel_func(
-            _gl_transform, self.n_jobs, verbose=False)
-        n_jobs = min(n_jobs, X.shape[-1])
-        with ProgressBar(X.shape[-1] * len(self.estimators_), mesg=mesg) as pb:
+            _gl_transform,
+            self.n_jobs,
+            max_jobs=X.shape[-1],
+            verbose=_verbose_safe_false(),
+        )
+
+        context = _create_progressbar_context(self, X, "Transforming")
+        with context as pb:
             y_pred = parallel(
                 p_func(self.estimators_, x_split, method, pb.subset(pb_idx))
                 for pb_idx, x_split in array_split_idx(
-                    X, n_jobs, axis=-1, n_per_split=len(self.estimators_)))
+                    X, n_jobs, axis=-1, n_per_split=len(self.estimators_)
+                )
+            )
 
         y_pred = np.concatenate(y_pred, axis=2)
+        if orig_method == "transform":
+            y_pred = y_pred.astype(X.dtype)
+        if orig_method == "predict_proba" and not is_nd:
+            y_pred = y_pred[:, 0, 0, :]
         return y_pred
 
     def transform(self, X):
@@ -465,7 +545,8 @@ class GeneralizingEstimator(SlidingEstimator):
         Xt : array, shape (n_samples, n_estimators, n_slices)
             The transformed values generated by each estimator.
         """
-        return self._transform(X, 'transform')
+        check_is_fitted(self)
+        return self._transform(X, "transform")
 
     def predict(self, X):
         """Predict each data slice with all possible estimators.
@@ -483,7 +564,7 @@ class GeneralizingEstimator(SlidingEstimator):
         y_pred : array, shape (n_samples, n_estimators, n_slices) | (n_samples, n_estimators, n_slices, n_targets)
             The predicted values for each estimator.
         """  # noqa: E501
-        return self._transform(X, 'predict')
+        return self._transform(X, "predict")
 
     def predict_proba(self, X):
         """Estimate probabilistic estimates of each data slice with all possible estimators.
@@ -505,7 +586,7 @@ class GeneralizingEstimator(SlidingEstimator):
         -----
         This requires ``base_estimator`` to have a ``predict_proba`` method.
         """  # noqa: E501
-        return self._transform(X, 'predict_proba')
+        return self._transform(X, "predict_proba")
 
     def decision_function(self, X):
         """Estimate distances of each data slice to all hyperplanes.
@@ -529,9 +610,8 @@ class GeneralizingEstimator(SlidingEstimator):
         This requires ``base_estimator`` to have a ``decision_function``
         method.
         """  # noqa: E501
-        return self._transform(X, 'decision_function')
+        return self._transform(X, "decision_function")
 
-    @verbose  # to use class value
     def score(self, X, y):
         """Score each of the estimators on the tested dimensions.
 
@@ -551,22 +631,26 @@ class GeneralizingEstimator(SlidingEstimator):
         score : array, shape (n_samples, n_estimators, n_slices)
             Score for each estimator / data slice couple.
         """  # noqa: E501
-        check_scoring = _get_check_scoring()
-        self._check_Xy(X)
+        X, _ = self._check_Xy(X, y)
         # For predictions/transforms the parallelization is across the data and
         # not across the estimators to avoid memory load.
-        mesg = 'Scoring %s' % (self.__class__.__name__,)
-        parallel, p_func, n_jobs = parallel_func(_gl_score, self.n_jobs,
-                                                 verbose=False)
-        n_jobs = min(n_jobs, X.shape[-1])
+        parallel, p_func, n_jobs = parallel_func(
+            _gl_score,
+            self.n_jobs,
+            max_jobs=X.shape[-1],
+            verbose=_verbose_safe_false(),
+        )
         scoring = check_scoring(self.base_estimator, self.scoring)
         y = _fix_auc(scoring, y)
-        with ProgressBar(X.shape[-1] * len(self.estimators_), mesg=mesg) as pb:
-            score = parallel(p_func(self.estimators_, scoring, x, y,
-                                    pb.subset(pb_idx))
-                             for pb_idx, x in array_split_idx(
-                                 X, n_jobs, axis=-1,
-                                 n_per_split=len(self.estimators_)))
+
+        context = _create_progressbar_context(self, X, "Scoring")
+        with context as pb:
+            score = parallel(
+                p_func(self.estimators_, scoring, x, y, pb.subset(pb_idx))
+                for pb_idx, x in array_split_idx(
+                    X, n_jobs, axis=-1, n_per_split=len(self.estimators_)
+                )
+            )
 
         score = np.concatenate(score, axis=1)
         return score
@@ -583,6 +667,10 @@ def _gl_transform(estimators, X, method, pb):
         The training input samples. For each data slice, a clone estimator
         is fitted independently. The feature dimension can be multidimensional
         e.g. X.shape = (n_samples, n_features_1, n_features_2, n_estimators)
+    method : str
+        The method to call for each estimator.
+    pb : instance of ProgressBar
+        The progress bar to update.
 
     Returns
     -------
@@ -606,6 +694,7 @@ def _gl_transform(estimators, X, method, pb):
         if ii == 0:
             y_pred = _gl_init_pred(_y_pred, X, len(estimators))
         y_pred[:, ii, ...] = _y_pred
+
         pb.update((ii + 1) * n_iter)
     return y_pred
 
@@ -614,8 +703,7 @@ def _gl_init_pred(y_pred, X, n_train):
     """Aux. function to GeneralizingEstimator to initialize y_pred."""
     n_sample, n_iter = X.shape[0], X.shape[-1]
     if y_pred.ndim == 3:
-        y_pred = np.zeros((n_sample, n_train, n_iter, y_pred.shape[-1]),
-                          y_pred.dtype)
+        y_pred = np.zeros((n_sample, n_train, n_iter, y_pred.shape[-1]), y_pred.dtype)
     else:
         y_pred = np.zeros((n_sample, n_train, n_iter), y_pred.dtype)
     return y_pred
@@ -639,6 +727,8 @@ def _gl_score(estimators, scoring, X, y, pb):
         X.shape = (n_samples, n_features_1, n_features_2, n_estimators)
     y : array, shape (n_samples,) | (n_samples, n_targets)
         The target values.
+    pb : instance of ProgressBar
+        The progress bar to update.
 
     Returns
     -------
@@ -656,21 +746,45 @@ def _gl_score(estimators, scoring, X, y, pb):
                 dtype = type(_score)
                 score = np.zeros(score_shape, dtype)
             score[ii, jj, ...] = _score
+
             pb.update(jj * len(estimators) + ii + 1)
     return score
 
 
 def _fix_auc(scoring, y):
-    from sklearn.preprocessing import LabelEncoder
     # This fixes sklearn's inability to compute roc_auc when y not in [0, 1]
     # scikit-learn/scikit-learn#6874
     if scoring is not None:
-        score_func = getattr(scoring, '_score_func', None)
-        kwargs = getattr(scoring, '_kwargs', {})
-        if (getattr(score_func, '__name__', '') == 'roc_auc_score' and
-                kwargs.get('multi_class', 'raise') == 'raise'):
+        score_func = getattr(scoring, "_score_func", None)
+        kwargs = getattr(scoring, "_kwargs", {})
+        if (
+            getattr(score_func, "__name__", "") == "roc_auc_score"
+            and kwargs.get("multi_class", "raise") == "raise"
+        ):
             if np.ndim(y) != 1 or len(set(y)) != 2:
-                raise ValueError('roc_auc scoring can only be computed for '
-                                 'two-class problems.')
+                raise ValueError(
+                    "roc_auc scoring can only be computed for two-class problems."
+                )
             y = LabelEncoder().fit_transform(y)
     return y
+
+
+def _create_progressbar_context(inst, X, message):
+    """Create a progress bar taking into account ``inst.verbose``."""
+    multiply = len(inst.estimators_) if isinstance(inst, GeneralizingEstimator) else 1
+    n_steps = X.shape[-1] * max(1, multiply)
+    mesg = f"{message} {inst.__class__.__name__}"
+
+    which_tqdm = "off" if not _check_verbose(inst.verbose) else None
+    context = ProgressBar(
+        n_steps, mesg=mesg, position=inst.position, which_tqdm=which_tqdm
+    )
+
+    return context
+
+
+def _check_verbose(verbose):
+    """Check if verbose is above or equal 'INFO' level."""
+    logging_level = _parse_verbose(verbose)
+    bool_verbose = logging_level <= logging.INFO
+    return bool_verbose
