@@ -1,25 +1,80 @@
-# -*- coding: utf-8 -*-
-# Authors: Mainak Jas <mainak@neuro.hut.fi>
-#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#          Romain Trachel <trachelr@gmail.com>
-#
+# Authors: The MNE-Python contributors.
 # License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
 
 import numpy as np
+from sklearn.base import BaseEstimator, TransformerMixin, check_array, clone
+from sklearn.preprocessing import RobustScaler, StandardScaler
+from sklearn.utils import check_X_y
+from sklearn.utils.validation import check_is_fitted
 
-from .mixin import TransformerMixin
-from .base import BaseEstimator
-
-from .. import pick_types
+from .._fiff.pick import (
+    _pick_data_channels,
+    _picks_by_type,
+    _picks_to_idx,
+    pick_info,
+)
+from ..cov import _check_scalings_user
+from ..epochs import BaseEpochs
 from ..filter import filter_data
 from ..time_frequency import psd_array_multitaper
-from ..utils import fill_doc, _check_option, _validate_type, verbose
-from ..io.pick import (pick_info, _pick_data_channels, _picks_by_type,
-                       _picks_to_idx)
-from ..cov import _check_scalings_user
+from ..utils import _check_option, _validate_type, check_version, fill_doc
+from ._fixes import validate_data  # TODO VERSION remove with sklearn 1.4+
 
 
-class _ConstantScaler():
+class MNETransformerMixin(TransformerMixin):
+    """TransformerMixin plus some helpers."""
+
+    def _check_data(
+        self,
+        epochs_data,
+        *,
+        y=None,
+        atleast_3d=True,
+        fit=False,
+        return_y=False,
+        multi_output=False,
+        check_n_features=True,
+    ):
+        # Sklearn calls asarray under the hood which works, but elsewhere they check for
+        # __len__ then look at the size of obj[0]... which is an epoch of shape (1, ...)
+        # rather than what they expect (shape (...)). So we explicitly get the NumPy
+        # array to make everyone happy.
+        if isinstance(epochs_data, BaseEpochs):
+            epochs_data = epochs_data.get_data(copy=False)
+        kwargs = dict(dtype=np.float64, allow_nd=True, order="C")
+        if check_version("sklearn", "1.4"):  # TODO VERSION sklearn 1.4+
+            kwargs["force_writeable"] = True
+        if hasattr(self, "n_features_in_") and check_n_features:
+            if y is None:
+                epochs_data = validate_data(
+                    self,
+                    epochs_data,
+                    **kwargs,
+                    reset=fit,
+                )
+            else:
+                epochs_data, y = validate_data(
+                    self,
+                    epochs_data,
+                    y,
+                    **kwargs,
+                    reset=fit,
+                )
+        elif y is None:
+            epochs_data = check_array(epochs_data, **kwargs)
+        else:
+            epochs_data, y = check_X_y(
+                X=epochs_data, y=y, multi_output=multi_output, **kwargs
+            )
+        if fit:
+            self.n_features_in_ = epochs_data.shape[1]
+        if atleast_3d:
+            epochs_data = np.atleast_3d(epochs_data)
+        return (epochs_data, y) if return_y else epochs_data
+
+
+class _ConstantScaler:
     """Scale channel types using constant values."""
 
     def __init__(self, info, scalings, do_scaling=True):
@@ -29,15 +84,17 @@ class _ConstantScaler():
 
     def fit(self, X, y=None):
         scalings = _check_scalings_user(self._scalings)
-        picks_by_type = _picks_by_type(pick_info(
-            self._info, _pick_data_channels(self._info, exclude=())))
+        picks_by_type = _picks_by_type(
+            pick_info(self._info, _pick_data_channels(self._info, exclude=()))
+        )
         std = np.ones(sum(len(p[1]) for p in picks_by_type))
         if X.shape[1] != len(std):
-            raise ValueError('info had %d data channels but X has %d channels'
-                             % (len(std), len(X)))
+            raise ValueError(
+                f"info had {len(std)} data channels but X has {len(X)} channels"
+            )
         if self._do_scaling:  # this is silly, but necessary for completeness
             for kind, picks in picks_by_type:
-                std[picks] = 1. / scalings[kind]
+                std[picks] = 1.0 / scalings[kind]
         self.std_ = std
         self.mean_ = np.zeros_like(std)
         return self
@@ -54,8 +111,9 @@ class _ConstantScaler():
 
 def _sklearn_reshape_apply(func, return_result, X, *args, **kwargs):
     """Reshape epochs and apply function."""
-    if not isinstance(X, np.ndarray):
-        raise ValueError("data should be an np.ndarray, got %s." % type(X))
+    _validate_type(X, np.ndarray, "X")
+    if X.size == 0:
+        return X.copy() if return_result else None
     orig_shape = X.shape
     X = np.reshape(X.transpose(0, 2, 1), (-1, orig_shape[1]))
     X = func(X, *args, **kwargs)
@@ -66,8 +124,8 @@ def _sklearn_reshape_apply(func, return_result, X, *args, **kwargs):
 
 
 @fill_doc
-class Scaler(TransformerMixin, BaseEstimator):
-    u"""Standardize channel data.
+class Scaler(MNETransformerMixin, BaseEstimator):
+    """Standardize channel data.
 
     This class scales data for each channel. It differs from scikit-learn
     classes (e.g., :class:`sklearn.preprocessing.StandardScaler`) in that
@@ -102,32 +160,12 @@ class Scaler(TransformerMixin, BaseEstimator):
         if ``scalings`` is a dict or None).
     """
 
-    def __init__(self, info=None, scalings=None, with_mean=True,
-                 with_std=True):  # noqa: D102
+    def __init__(self, info=None, scalings=None, with_mean=True, with_std=True):
         self.info = info
         self.with_mean = with_mean
         self.with_std = with_std
         self.scalings = scalings
 
-        if not (scalings is None or isinstance(scalings, (dict, str))):
-            raise ValueError('scalings type should be dict, str, or None, '
-                             'got %s' % type(scalings))
-        if isinstance(scalings, str):
-            _check_option('scalings', scalings, ['mean', 'median'])
-        if scalings is None or isinstance(scalings, dict):
-            if info is None:
-                raise ValueError('Need to specify "info" if scalings is'
-                                 '%s' % type(scalings))
-            self._scaler = _ConstantScaler(info, scalings, self.with_std)
-        elif scalings == 'mean':
-            from sklearn.preprocessing import StandardScaler
-            self._scaler = StandardScaler(
-                with_mean=self.with_mean, with_std=self.with_std)
-        else:  # scalings == 'median':
-            from sklearn.preprocessing import RobustScaler
-            self._scaler = RobustScaler(
-                with_centering=self.with_mean, with_scaling=self.with_std)
-
     def fit(self, epochs_data, y=None):
         """Standardize data across channels.
 
@@ -143,11 +181,30 @@ class Scaler(TransformerMixin, BaseEstimator):
         self : instance of Scaler
             The modified instance.
         """
-        _validate_type(epochs_data, np.ndarray, 'epochs_data')
-        if epochs_data.ndim == 2:
-            epochs_data = epochs_data[..., np.newaxis]
+        epochs_data = self._check_data(epochs_data, y=y, fit=True, multi_output=True)
         assert epochs_data.ndim == 3, epochs_data.shape
-        _sklearn_reshape_apply(self._scaler.fit, False, epochs_data, y=y)
+
+        _validate_type(self.scalings, (dict, str, type(None)), "scalings")
+        if isinstance(self.scalings, str):
+            _check_option(
+                "scalings", self.scalings, ["mean", "median"], extra="when str"
+            )
+        if self.scalings is None or isinstance(self.scalings, dict):
+            if self.info is None:
+                raise ValueError(
+                    f'Need to specify "info" if scalings is {type(self.scalings)}'
+                )
+            self.scaler_ = _ConstantScaler(self.info, self.scalings, self.with_std)
+        elif self.scalings == "mean":
+            self.scaler_ = StandardScaler(
+                with_mean=self.with_mean, with_std=self.with_std
+            )
+        else:  # scalings == 'median':
+            self.scaler_ = RobustScaler(
+                with_centering=self.with_mean, with_scaling=self.with_std
+            )
+
+        _sklearn_reshape_apply(self.scaler_.fit, False, epochs_data, y=y)
         return self
 
     def transform(self, epochs_data):
@@ -168,14 +225,14 @@ class Scaler(TransformerMixin, BaseEstimator):
         This function makes a copy of the data before the operations and the
         memory usage may be large with big data.
         """
-        _validate_type(epochs_data, np.ndarray, 'epochs_data')
+        check_is_fitted(self, "scaler_")
+        epochs_data = self._check_data(epochs_data, atleast_3d=False)
         if epochs_data.ndim == 2:  # can happen with SlidingEstimator
             if self.info is not None:
-                assert len(self.info['ch_names']) == epochs_data.shape[1]
+                assert len(self.info["ch_names"]) == epochs_data.shape[1]
             epochs_data = epochs_data[..., np.newaxis]
         assert epochs_data.ndim == 3, epochs_data.shape
-        return _sklearn_reshape_apply(self._scaler.transform, True,
-                                      epochs_data)
+        return _sklearn_reshape_apply(self.scaler_.transform, True, epochs_data)
 
     def fit_transform(self, epochs_data, y=None):
         """Fit to data, then transform it.
@@ -208,7 +265,7 @@ class Scaler(TransformerMixin, BaseEstimator):
 
         Parameters
         ----------
-        epochs_data : array, shape (n_epochs, n_channels, n_times)
+        epochs_data : array, shape ([n_epochs, ]n_channels, n_times)
             The data.
 
         Returns
@@ -221,12 +278,20 @@ class Scaler(TransformerMixin, BaseEstimator):
         This function makes a copy of the data before the operations and the
         memory usage may be large with big data.
         """
+        epochs_data = self._check_data(epochs_data, atleast_3d=False)
+        squeeze = False
+        # Can happen with CSP
+        if epochs_data.ndim == 2:
+            squeeze = True
+            epochs_data = epochs_data[..., np.newaxis]
         assert epochs_data.ndim == 3, epochs_data.shape
-        return _sklearn_reshape_apply(self._scaler.inverse_transform, True,
-                                      epochs_data)
+        out = _sklearn_reshape_apply(self.scaler_.inverse_transform, True, epochs_data)
+        if squeeze:
+            out = out[..., 0]
+        return out
 
 
-class Vectorizer(TransformerMixin):
+class Vectorizer(MNETransformerMixin, BaseEstimator):
     """Transform n-dimensional array into 2D array of n_samples by n_features.
 
     This class reshapes an n-dimensional array into an n_samples * n_features
@@ -239,8 +304,10 @@ class Vectorizer(TransformerMixin):
 
     Examples
     --------
-    clf = make_pipeline(SpatialFilter(), _XdawnTransformer(), Vectorizer(),
-                        LogisticRegression())
+    >>> from sklearn.linear_model import LogisticRegression
+    >>> from sklearn.pipeline import make_pipeline
+    >>> from sklearn.preprocessing import StandardScaler
+    >>> clf = make_pipeline(Vectorizer(), StandardScaler(), LogisticRegression())
     """
 
     def fit(self, X, y=None):
@@ -261,7 +328,7 @@ class Vectorizer(TransformerMixin):
         self : instance of Vectorizer
             Return the modified instance.
         """
-        X = np.asarray(X)
+        X = self._check_data(X, y=y, atleast_3d=False, fit=True, check_n_features=False)
         self.features_shape_ = X.shape[1:]
         return self
 
@@ -281,10 +348,9 @@ class Vectorizer(TransformerMixin):
         X : array, shape (n_samples, n_features)
             The transformed data.
         """
-        X = np.asarray(X)
+        X = self._check_data(X, atleast_3d=False)
         if X.shape[1:] != self.features_shape_:
-            raise ValueError("Shape of X used in fit and transform must be "
-                             "same")
+            raise ValueError("Shape of X used in fit and transform must be same")
         return X.reshape(len(X), -1)
 
     def fit_transform(self, X, y=None):
@@ -321,15 +387,16 @@ class Vectorizer(TransformerMixin):
             The data transformed into shape as used in fit. The first
             dimension is of length n_samples.
         """
-        X = np.asarray(X)
+        X = self._check_data(X, atleast_3d=False, check_n_features=False)
         if X.ndim not in (2, 3):
-            raise ValueError("X should be of 2 or 3 dimensions but has shape "
-                             "%s" % (X.shape,))
+            raise ValueError(
+                f"X should be of 2 or 3 dimensions but has shape {X.shape}"
+            )
         return X.reshape(X.shape[:-1] + self.features_shape_)
 
 
 @fill_doc
-class PSDEstimator(TransformerMixin):
+class PSDEstimator(MNETransformerMixin, BaseEstimator):
     """Compute power spectral density (PSD) using a multi-taper method.
 
     Parameters
@@ -351,17 +418,26 @@ class PSDEstimator(TransformerMixin):
     n_jobs : int
         Number of parallel jobs to use (only used if adaptive=True).
     %(normalization)s
-    %(verbose)s
 
     See Also
     --------
-    mne.time_frequency.psd_multitaper
+    mne.time_frequency.psd_array_multitaper
+    mne.io.Raw.compute_psd
+    mne.Epochs.compute_psd
+    mne.Evoked.compute_psd
     """
 
-    @verbose
-    def __init__(self, sfreq=2 * np.pi, fmin=0, fmax=np.inf, bandwidth=None,
-                 adaptive=False, low_bias=True, n_jobs=None,
-                 normalization='length', *, verbose=None):  # noqa: D102
+    def __init__(
+        self,
+        sfreq=2 * np.pi,
+        fmin=0,
+        fmax=np.inf,
+        bandwidth=None,
+        adaptive=False,
+        low_bias=True,
+        n_jobs=None,
+        normalization="length",
+    ):
         self.sfreq = sfreq
         self.fmin = fmin
         self.fmax = fmax
@@ -371,7 +447,7 @@ class PSDEstimator(TransformerMixin):
         self.n_jobs = n_jobs
         self.normalization = normalization
 
-    def fit(self, epochs_data, y):
+    def fit(self, epochs_data, y=None):
         """Compute power spectral density (PSD) using a multi-taper method.
 
         Parameters
@@ -386,10 +462,8 @@ class PSDEstimator(TransformerMixin):
         self : instance of PSDEstimator
             The modified instance.
         """
-        if not isinstance(epochs_data, np.ndarray):
-            raise ValueError("epochs_data should be of type ndarray (got %s)."
-                             % type(epochs_data))
-
+        self._check_data(epochs_data, y=y, fit=True)
+        self.fitted_ = True  # sklearn compliance
         return self
 
     def transform(self, epochs_data):
@@ -405,19 +479,23 @@ class PSDEstimator(TransformerMixin):
         psd : array, shape (n_signals, n_freqs) or (n_freqs,)
             The computed PSD.
         """
-        if not isinstance(epochs_data, np.ndarray):
-            raise ValueError("epochs_data should be of type ndarray (got %s)."
-                             % type(epochs_data))
+        epochs_data = self._check_data(epochs_data)
         psd, _ = psd_array_multitaper(
-            epochs_data, sfreq=self.sfreq, fmin=self.fmin, fmax=self.fmax,
-            bandwidth=self.bandwidth, adaptive=self.adaptive,
-            low_bias=self.low_bias, normalization=self.normalization,
-            n_jobs=self.n_jobs)
+            epochs_data,
+            sfreq=self.sfreq,
+            fmin=self.fmin,
+            fmax=self.fmax,
+            bandwidth=self.bandwidth,
+            adaptive=self.adaptive,
+            low_bias=self.low_bias,
+            normalization=self.normalization,
+            n_jobs=self.n_jobs,
+        )
         return psd
 
 
 @fill_doc
-class FilterEstimator(TransformerMixin):
+class FilterEstimator(MNETransformerMixin, BaseEstimator):
     """Estimator to filter RtEpochs.
 
     Applies a zero-phase low-pass, high-pass, band-pass, or band-stop
@@ -447,14 +525,12 @@ class FilterEstimator(TransformerMixin):
         Number of jobs to run in parallel.
         Can be 'cuda' if ``cupy`` is installed properly and method='fir'.
     method : str
-        'fir' will use overlap-add FIR filtering, 'iir' will use IIR
-        forward-backward filtering (via filtfilt).
+        'fir' will use overlap-add FIR filtering, 'iir' will use IIR filtering.
     iir_params : dict | None
         Dictionary of parameters to use for IIR filtering.
         See mne.filter.construct_iir_filter for details. If iir_params
         is None and method="iir", 4th order Butterworth will be used.
     %(fir_design)s
-    %(verbose)s
 
     See Also
     --------
@@ -462,20 +538,29 @@ class FilterEstimator(TransformerMixin):
 
     Notes
     -----
-    This is primarily meant for use in conjunction with
-    :class:`mne_realtime.RtEpochs`. In general it is not recommended in a
-    normal processing pipeline as it may result in edge artifacts. Use with
-    caution.
+    This is primarily meant for use in realtime applications.
+    In general it is not recommended in a normal processing pipeline as it may result
+    in edge artifacts. Use with caution.
     """
 
-    def __init__(self, info, l_freq, h_freq, picks=None, filter_length='auto',
-                 l_trans_bandwidth='auto', h_trans_bandwidth='auto',
-                 n_jobs=None, method='fir', iir_params=None,
-                 fir_design='firwin', *, verbose=None):  # noqa: D102
+    def __init__(
+        self,
+        info,
+        l_freq,
+        h_freq,
+        picks=None,
+        filter_length="auto",
+        l_trans_bandwidth="auto",
+        h_trans_bandwidth="auto",
+        n_jobs=None,
+        method="fir",
+        iir_params=None,
+        fir_design="firwin",
+    ):
         self.info = info
         self.l_freq = l_freq
         self.h_freq = h_freq
-        self.picks = _picks_to_idx(info, picks)
+        self.picks = picks
         self.filter_length = filter_length
         self.l_trans_bandwidth = l_trans_bandwidth
         self.h_trans_bandwidth = h_trans_bandwidth
@@ -499,38 +584,27 @@ class FilterEstimator(TransformerMixin):
         self : instance of FilterEstimator
             The modified instance.
         """
-        if not isinstance(epochs_data, np.ndarray):
-            raise ValueError("epochs_data should be of type ndarray (got %s)."
-                             % type(epochs_data))
-
-        if self.picks is None:
-            self.picks = pick_types(self.info, meg=True, eeg=True,
-                                    ref_meg=False, exclude=[])
+        self.picks_ = _picks_to_idx(self.info, self.picks)
+        self._check_data(epochs_data, y=y, fit=True)
 
         if self.l_freq == 0:
             self.l_freq = None
-        if self.h_freq is not None and self.h_freq > (self.info['sfreq'] / 2.):
-            self.h_freq = None
-        if self.l_freq is not None and not isinstance(self.l_freq, float):
-            self.l_freq = float(self.l_freq)
-        if self.h_freq is not None and not isinstance(self.h_freq, float):
-            self.h_freq = float(self.h_freq)
 
-        if self.info['lowpass'] is None or (self.h_freq is not None and
-                                            (self.l_freq is None or
-                                             self.l_freq < self.h_freq) and
-                                            self.h_freq <
-                                            self.info['lowpass']):
+        if self.info["lowpass"] is None or (
+            self.h_freq is not None
+            and (self.l_freq is None or self.l_freq < self.h_freq)
+            and self.h_freq < self.info["lowpass"]
+        ):
             with self.info._unlock():
-                self.info['lowpass'] = self.h_freq
+                self.info["lowpass"] = self.h_freq
 
-        if self.info['highpass'] is None or (self.l_freq is not None and
-                                             (self.h_freq is None or
-                                              self.l_freq < self.h_freq) and
-                                             self.l_freq >
-                                             self.info['highpass']):
+        if self.info["highpass"] is None or (
+            self.l_freq is not None
+            and (self.h_freq is None or self.l_freq < self.h_freq)
+            and self.l_freq > self.info["highpass"]
+        ):
             with self.info._unlock():
-                self.info['highpass'] = self.l_freq
+                self.info["highpass"] = self.l_freq
 
         return self
 
@@ -547,19 +621,25 @@ class FilterEstimator(TransformerMixin):
         X : array, shape (n_epochs, n_channels, n_times)
             The data after filtering.
         """
-        if not isinstance(epochs_data, np.ndarray):
-            raise ValueError("epochs_data should be of type ndarray (got %s)."
-                             % type(epochs_data))
-        epochs_data = np.atleast_3d(epochs_data)
         return filter_data(
-            epochs_data, self.info['sfreq'], self.l_freq, self.h_freq,
-            self.picks, self.filter_length, self.l_trans_bandwidth,
-            self.h_trans_bandwidth, method=self.method,
-            iir_params=self.iir_params, n_jobs=self.n_jobs, copy=False,
-            fir_design=self.fir_design, verbose=False)
+            self._check_data(epochs_data),
+            self.info["sfreq"],
+            self.l_freq,
+            self.h_freq,
+            self.picks_,
+            self.filter_length,
+            self.l_trans_bandwidth,
+            self.h_trans_bandwidth,
+            method=self.method,
+            iir_params=self.iir_params,
+            n_jobs=self.n_jobs,
+            copy=False,
+            fir_design=self.fir_design,
+            verbose=False,
+        )
 
 
-class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
+class UnsupervisedSpatialFilter(MNETransformerMixin, BaseEstimator):
     """Use unsupervised spatial filtering across time and samples.
 
     Parameters
@@ -571,17 +651,7 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
         (e.g. epochs).
     """
 
-    def __init__(self, estimator, average=False):  # noqa: D102
-        # XXX: Use _check_estimator #3381
-        for attr in ('fit', 'transform', 'fit_transform'):
-            if not hasattr(estimator, attr):
-                raise ValueError('estimator must be a scikit-learn '
-                                 'transformer, missing %s method' % attr)
-
-        if not isinstance(average, bool):
-            raise ValueError("average parameter must be of bool type, got "
-                             "%s instead" % type(bool))
-
+    def __init__(self, estimator, average=False):
         self.estimator = estimator
         self.average = average
 
@@ -600,14 +670,25 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
         self : instance of UnsupervisedSpatialFilter
             Return the modified instance.
         """
+        # sklearn.utils.estimator_checks.check_estimator(self.estimator) is probably
+        # too strict for us, given that we don't fully adhere yet, so just check attrs
+        for attr in ("fit", "transform", "fit_transform"):
+            if not hasattr(self.estimator, attr):
+                raise ValueError(
+                    "estimator must be a scikit-learn "
+                    f"transformer, missing {attr} method"
+                )
+        _validate_type(self.average, bool, "average")
+        X = self._check_data(X, y=y, fit=True)
         if self.average:
             X = np.mean(X, axis=0).T
         else:
             n_epochs, n_channels, n_times = X.shape
             # trial as time samples
-            X = np.transpose(X, (1, 0, 2)).reshape((n_channels, n_epochs *
-                                                    n_times)).T
-        self.estimator.fit(X)
+            X = np.transpose(X, (1, 0, 2)).reshape((n_channels, n_epochs * n_times)).T
+
+        self.estimator_ = clone(self.estimator)
+        self.estimator_.fit(X)
         return self
 
     def fit_transform(self, X, y=None):
@@ -640,7 +721,9 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
         X : array, shape (n_epochs, n_channels, n_times)
             The transformed data.
         """
-        return self._apply_method(X, 'transform')
+        check_is_fitted(self.estimator_)
+        X = self._check_data(X)
+        return self._apply_method(X, "transform")
 
     def inverse_transform(self, X):
         """Inverse transform the data to its original space.
@@ -655,7 +738,7 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
         X : array, shape (n_epochs, n_channels, n_times)
             The transformed data.
         """
-        return self._apply_method(X, 'inverse_transform')
+        return self._apply_method(X, "inverse_transform")
 
     def _apply_method(self, X, method):
         """Vectorize time samples as trials, apply method and reshape back.
@@ -675,7 +758,7 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
         X = np.transpose(X, [1, 0, 2])
         X = np.reshape(X, [n_channels, n_epochs * n_times]).T
         # apply method
-        method = getattr(self.estimator, method)
+        method = getattr(self.estimator_, method)
         X = method(X)
         # put it back to n_epochs, n_dimensions
         X = np.reshape(X.T, [-1, n_epochs, n_times]).transpose([1, 0, 2])
@@ -683,7 +766,7 @@ class UnsupervisedSpatialFilter(TransformerMixin, BaseEstimator):
 
 
 @fill_doc
-class TemporalFilter(TransformerMixin):
+class TemporalFilter(MNETransformerMixin, BaseEstimator):
     """Estimator to filter data array along the last dimension.
 
     Applies a zero-phase low-pass, high-pass, band-pass, or band-stop
@@ -757,7 +840,6 @@ class TemporalFilter(TransformerMixin):
         attenuation using fewer samples than "firwin2".
 
         .. versionadded:: 0.15
-    %(verbose)s
 
     See Also
     --------
@@ -766,12 +848,20 @@ class TemporalFilter(TransformerMixin):
     mne.filter.filter_data
     """
 
-    @verbose
-    def __init__(self, l_freq=None, h_freq=None, sfreq=1.0,
-                 filter_length='auto', l_trans_bandwidth='auto',
-                 h_trans_bandwidth='auto', n_jobs=None, method='fir',
-                 iir_params=None, fir_window='hamming', fir_design='firwin',
-                 *, verbose=None):  # noqa: D102
+    def __init__(
+        self,
+        l_freq=None,
+        h_freq=None,
+        sfreq=1.0,
+        filter_length="auto",
+        l_trans_bandwidth="auto",
+        h_trans_bandwidth="auto",
+        n_jobs=None,
+        method="fir",
+        iir_params=None,
+        fir_window="hamming",
+        fir_design="firwin",
+    ):
         self.l_freq = l_freq
         self.h_freq = h_freq
         self.sfreq = sfreq
@@ -784,16 +874,12 @@ class TemporalFilter(TransformerMixin):
         self.fir_window = fir_window
         self.fir_design = fir_design
 
-        if not isinstance(self.n_jobs, int) and self.n_jobs == 'cuda':
-            raise ValueError('n_jobs must be int or "cuda", got %s instead.'
-                             % type(self.n_jobs))
-
     def fit(self, X, y=None):
         """Do nothing (for scikit-learn compatibility purposes).
 
         Parameters
         ----------
-        X : array, shape (n_epochs, n_channels, n_times) or or shape (n_channels, n_times)
+        X : array, shape ([n_epochs, ]n_channels, n_times)
             The data to be filtered over the last dimension. The channels
             dimension can be zero when passing a 2D array.
         y : None
@@ -803,7 +889,9 @@ class TemporalFilter(TransformerMixin):
         -------
         self : instance of TemporalFilter
             The modified instance.
-        """  # noqa: E501
+        """
+        self.fitted_ = True  # sklearn compliance
+        self._check_data(X, y=y, atleast_3d=False, fit=True)
         return self
 
     def transform(self, X):
@@ -811,7 +899,7 @@ class TemporalFilter(TransformerMixin):
 
         Parameters
         ----------
-        X : array, shape (n_epochs, n_channels, n_times) or shape (n_channels, n_times)
+        X : array, shape ([n_epochs, ]n_channels, n_times)
             The data to be filtered over the last dimension. The channels
             dimension can be zero when passing a 2D array.
 
@@ -820,19 +908,30 @@ class TemporalFilter(TransformerMixin):
         X : array
             The data after filtering.
         """  # noqa: E501
+        X = self._check_data(X, atleast_3d=False)
         X = np.atleast_2d(X)
 
         if X.ndim > 3:
-            raise ValueError("Array must be of at max 3 dimensions instead "
-                             "got %s dimensional matrix" % (X.ndim))
+            raise ValueError(
+                "Array must be of at max 3 dimensions instead "
+                f"got {X.ndim} dimensional matrix"
+            )
 
         shape = X.shape
         X = X.reshape(-1, shape[-1])
-        X = filter_data(X, self.sfreq, self.l_freq, self.h_freq,
-                        filter_length=self.filter_length,
-                        l_trans_bandwidth=self.l_trans_bandwidth,
-                        h_trans_bandwidth=self.h_trans_bandwidth,
-                        n_jobs=self.n_jobs, method=self.method,
-                        iir_params=self.iir_params, copy=False,
-                        fir_window=self.fir_window, fir_design=self.fir_design)
+        X = filter_data(
+            X,
+            self.sfreq,
+            self.l_freq,
+            self.h_freq,
+            filter_length=self.filter_length,
+            l_trans_bandwidth=self.l_trans_bandwidth,
+            h_trans_bandwidth=self.h_trans_bandwidth,
+            n_jobs=self.n_jobs,
+            method=self.method,
+            iir_params=self.iir_params,
+            copy=False,
+            fir_window=self.fir_window,
+            fir_design=self.fir_design,
+        )
         return X.reshape(shape)
