@@ -1,11 +1,6 @@
-# -*- coding: utf-8 -*-
-# Authors: Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
-#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
-#          Eric Larson <larsoner@uw.edu>
-#          Mark Wronkiewicz <wronk@uw.edu>
-#
+# Authors: The MNE-Python contributors.
 # License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
 
 # The computations in this code were primarily derived from Matti Hämäläinen's
 # C code.
@@ -16,37 +11,38 @@
 # 2) EEG and MEG: forward solutions for inverse methods. Mosher, Leahy, and
 #        Lewis, 1999. Generalized discussion of forward solutions.
 
-import numpy as np
 from copy import deepcopy
 
-from ..fixes import jit, bincount
-from ..io.compensator import get_current_comp, make_compensator
-from ..io.constants import FIFF, FWD
-from ..io.pick import pick_types
-from ..parallel import parallel_func
-from ..surface import _project_onto_surface, _jit_cross
-from ..transforms import apply_trans
-from ..utils import logger, verbose, _pl, warn, fill_doc
+import numpy as np
 
+from .._fiff.constants import FIFF
+from ..bem import _import_openmeeg, _make_openmeeg_geometry
+from ..fixes import bincount, jit
+from ..parallel import parallel_func
+from ..surface import _jit_cross, _project_onto_surface
+from ..transforms import apply_trans, invert_transform
+from ..utils import _check_option, _pl, fill_doc, logger, verbose, warn
 
 # #############################################################################
 # COIL SPECIFICATION AND FIELD COMPUTATION MATRIX
 
+
 def _dup_coil_set(coils, coord_frame, t):
     """Make a duplicate."""
-    if t is not None and coord_frame != t['from']:
-        raise RuntimeError('transformation frame does not match the coil set')
+    if t is not None and coord_frame != t["from"]:
+        raise RuntimeError("transformation frame does not match the coil set")
     coils = deepcopy(coils)
     if t is not None:
-        coord_frame = t['to']
+        coord_frame = t["to"]
         for coil in coils:
-            for key in ('ex', 'ey', 'ez'):
+            assert isinstance(coil, dict), f"Coil must be a dict, got {type(coil)}"
+            for key in ("ex", "ey", "ez"):
                 if key in coil:
-                    coil[key] = apply_trans(t['trans'], coil[key], False)
-            coil['r0'] = apply_trans(t['trans'], coil['r0'])
-            coil['rmag'] = apply_trans(t['trans'], coil['rmag'])
-            coil['cosmag'] = apply_trans(t['trans'], coil['cosmag'], False)
-            coil['coord_frame'] = t['to']
+                    coil[key] = apply_trans(t["trans"], coil[key], False)
+            coil["r0"] = apply_trans(t["trans"], coil["r0"])
+            coil["rmag"] = apply_trans(t["trans"], coil["rmag"])
+            coil["cosmag"] = apply_trans(t["trans"], coil["cosmag"], False)
+            coil["coord_frame"] = t["to"]
     return coils, coord_frame
 
 
@@ -55,10 +51,9 @@ def _check_coil_frame(coils, coord_frame, bem):
     if coord_frame != FIFF.FIFFV_COORD_MRI:
         if coord_frame == FIFF.FIFFV_COORD_HEAD:
             # Make a transformed duplicate
-            coils, coord_Frame = _dup_coil_set(coils, coord_frame,
-                                               bem['head_mri_t'])
+            coils, coord_frame = _dup_coil_set(coils, coord_frame, bem["head_mri_t"])
         else:
-            raise RuntimeError('Bad coil coordinate frame %s' % coord_frame)
+            raise RuntimeError(f"Bad coil coordinate frame {coord_frame}")
     return coils, coord_frame
 
 
@@ -89,12 +84,18 @@ def _lin_field_coeff(surf, mult, rmags, cosmags, ws, bins, n_jobs):
         Linear coefficients with lead fields for each BEM vertex on each sensor
         (?)
     """
-    parallel, p_fun, _ = parallel_func(_do_lin_field_coeff, n_jobs)
+    parallel, p_fun, n_jobs = parallel_func(
+        _do_lin_field_coeff, n_jobs, max_jobs=len(surf["tris"])
+    )
     nas = np.array_split
-    coeffs = parallel(p_fun(surf['rr'], t, tn, ta, rmags, cosmags, ws, bins)
-                      for t, tn, ta in zip(nas(surf['tris'], n_jobs),
-                                           nas(surf['tri_nn'], n_jobs),
-                                           nas(surf['tri_area'], n_jobs)))
+    coeffs = parallel(
+        p_fun(surf["rr"], t, tn, ta, rmags, cosmags, ws, bins)
+        for t, tn, ta in zip(
+            nas(surf["tris"], n_jobs),
+            nas(surf["tri_nn"], n_jobs),
+            nas(surf["tri_area"], n_jobs),
+        )
+    )
     return mult * np.sum(coeffs, axis=0)
 
 
@@ -155,22 +156,21 @@ def _do_lin_field_coeff(bem_rr, tris, tn, ta, rmags, cosmags, ws, bins):
         for ti in range(3):
             x = np.sum(c[:, ti], axis=-1)
             x /= den[:, tri[ti]] / tri_area
-            coeff[:, tri[ti]] += \
-                bincount(bins, weights=x, minlength=bins[-1] + 1)
+            coeff[:, tri[ti]] += bincount(bins, weights=x, minlength=bins[-1] + 1)
     return coeff
 
 
 def _concatenate_coils(coils):
     """Concatenate MEG coil parameters."""
-    rmags = np.concatenate([coil['rmag'] for coil in coils])
-    cosmags = np.concatenate([coil['cosmag'] for coil in coils])
-    ws = np.concatenate([coil['w'] for coil in coils])
-    n_int = np.array([len(coil['rmag']) for coil in coils])
+    rmags = np.concatenate([coil["rmag"] for coil in coils])
+    cosmags = np.concatenate([coil["cosmag"] for coil in coils])
+    ws = np.concatenate([coil["w"] for coil in coils])
+    n_int = np.array([len(coil["rmag"]) for coil in coils])
     if n_int[-1] == 0:
         # We assume each sensor has at least one integration point,
         # which should be a safe assumption. But let's check it here, since
         # our code elsewhere relies on bins[-1] + 1 being the number of sensors
-        raise RuntimeError('not supported')
+        raise RuntimeError("not supported")
     bins = np.repeat(np.arange(len(n_int)), n_int)
     return rmags, cosmags, ws, bins
 
@@ -209,8 +209,8 @@ def _bem_specify_coils(bem, coils, coord_frame, mults, n_jobs):
     # Process each of the surfaces
     rmags, cosmags, ws, bins = _triage_coils(coils)
     del coils
-    lens = np.cumsum(np.r_[0, [len(s['rr']) for s in bem['surfs']]])
-    sol = np.zeros((bins[-1] + 1, bem['solution'].shape[1]))
+    lens = np.cumsum(np.r_[0, [len(s["rr"]) for s in bem["surfs"]]])
+    sol = np.zeros((bins[-1] + 1, bem["solution"].shape[1]))
 
     lims = np.concatenate([np.arange(0, sol.shape[0], 100), [sol.shape[0]]])
     # Put through the bem (in channel-based chunks to save memory)
@@ -218,10 +218,11 @@ def _bem_specify_coils(bem, coils, coord_frame, mults, n_jobs):
         mask = np.logical_and(bins >= start, bins < stop)
         r, c, w, b = rmags[mask], cosmags[mask], ws[mask], bins[mask] - start
         # Compute coeffs for each surface, one at a time
-        for o1, o2, surf, mult in zip(lens[:-1], lens[1:],
-                                      bem['surfs'], bem['field_mult']):
+        for o1, o2, surf, mult in zip(
+            lens[:-1], lens[1:], bem["surfs"], bem["field_mult"]
+        ):
             coeff = _lin_field_coeff(surf, mult, r, c, w, b, n_jobs)
-            sol[start:stop] += np.dot(coeff, bem['solution'][o1:o2])
+            sol[start:stop] += np.dot(coeff, bem["solution"][o1:o2])
     sol *= mults
     return sol
 
@@ -243,53 +244,28 @@ def _bem_specify_els(bem, els, mults):
     sol : ndarray, shape (n_EEG_sensors, n_BEM_vertices)
         EEG solution
     """
-    sol = np.zeros((len(els), bem['solution'].shape[1]))
-    scalp = bem['surfs'][0]
+    sol = np.zeros((len(els), bem["solution"].shape[1]))
+    scalp = bem["surfs"][0]
 
     # Operate on all integration points for all electrodes (in MRI coords)
-    rrs = np.concatenate([apply_trans(bem['head_mri_t']['trans'], el['rmag'])
-                          for el in els], axis=0)
-    ws = np.concatenate([el['w'] for el in els])
+    rrs = np.concatenate(
+        [apply_trans(bem["head_mri_t"]["trans"], el["rmag"]) for el in els], axis=0
+    )
+    ws = np.concatenate([el["w"] for el in els])
     tri_weights, tri_idx = _project_onto_surface(rrs, scalp)
     tri_weights *= ws[:, np.newaxis]
-    weights = np.matmul(tri_weights[:, np.newaxis],
-                        bem['solution'][scalp['tris'][tri_idx]])[:, 0]
+    weights = np.matmul(
+        tri_weights[:, np.newaxis], bem["solution"][scalp["tris"][tri_idx]]
+    )[:, 0]
     # there are way more vertices than electrodes generally, so let's iterate
     # over the electrodes
-    edges = np.concatenate([[0], np.cumsum([len(el['w']) for el in els])])
+    edges = np.concatenate([[0], np.cumsum([len(el["w"]) for el in els])])
     for ii, (start, stop) in enumerate(zip(edges[:-1], edges[1:])):
         sol[ii] = weights[start:stop].sum(0)
     sol *= mults
     return sol
 
 
-# #############################################################################
-# COMPENSATION
-
-def _make_ctf_comp_coils(info, coils):
-    """Get the correct compensator for CTF coils."""
-    # adapted from mne_make_ctf_comp() from mne_ctf_comp.c
-    logger.info('Setting up compensation data...')
-    comp_num = get_current_comp(info)
-    if comp_num is None or comp_num == 0:
-        logger.info('    No compensation set. Nothing more to do.')
-        return None
-
-    # Need to meaningfully populate comp['set'] dict a.k.a. compset
-    n_comp_ch = sum([c['kind'] == FIFF.FIFFV_MEG_CH for c in info['chs']])
-    logger.info('    %d out of %d channels have the compensation set.'
-                % (n_comp_ch, len(coils)))
-
-    # Find the desired compensation data matrix
-    compensator = make_compensator(info, 0, comp_num, True)
-    logger.info('    Desired compensation data (%s) found.' % comp_num)
-    logger.info('    All compensation channels found.')
-    logger.info('    Preselector created.')
-    logger.info('    Compensation data matrix created.')
-    logger.info('    Postselector created.')
-    return compensator
-
-
 # #############################################################################
 # BEM COMPUTATION
 
@@ -330,7 +306,7 @@ def _bem_inf_pots(mri_rr, bem_rr, mri_Q=None):
         this_diff = bem_rr - rr
         diff_norm = np.sum(this_diff * this_diff, axis=1)
         diff_norm *= np.sqrt(diff_norm)
-        diff_norm[diff_norm == 0] = 1.
+        diff_norm[diff_norm == 0] = 1.0
         if mri_Q is not None:
             this_diff = np.dot(this_diff, mri_Q.T)
         this_diff /= diff_norm.reshape(-1, 1)
@@ -338,6 +314,7 @@ def _bem_inf_pots(mri_rr, bem_rr, mri_Q=None):
 
     return diff
 
+
 # This function has been refactored to process all points simultaneously
 # def _bem_inf_field(rd, Q, rp, d):
 # """Infinite-medium magnetic field. See (7) in Mosher, 1999"""
@@ -398,8 +375,7 @@ def _bem_inf_fields(rr, rmag, cosmag):
 
 
 @fill_doc
-def _bem_pot_or_field(rr, mri_rr, mri_Q, coils, solution, bem_rr, n_jobs,
-                      coil_type):
+def _bem_pot_or_field(rr, mri_rr, mri_Q, coils, solution, bem_rr, n_jobs, coil_type):
     """Calculate the magnetic field or electric potential forward solution.
 
     The code is very similar between EEG and MEG potentials, so combine them.
@@ -432,21 +408,25 @@ def _bem_pot_or_field(rr, mri_rr, mri_Q, coils, solution, bem_rr, n_jobs,
     # Both MEG and EEG have the inifinite-medium potentials
     # This could be just vectorized, but eats too much memory, so instead we
     # reduce memory by chunking within _do_inf_pots and parallelize, too:
-    parallel, p_fun, _ = parallel_func(_do_inf_pots, n_jobs)
+    parallel, p_fun, n_jobs = parallel_func(_do_inf_pots, n_jobs, max_jobs=len(rr))
     nas = np.array_split
-    B = np.sum(parallel(p_fun(mri_rr, sr.copy(), np.ascontiguousarray(mri_Q),
-                              np.array(sol))  # copy and contig
-                        for sr, sol in zip(nas(bem_rr, n_jobs),
-                                           nas(solution.T, n_jobs))), axis=0)
+    B = np.sum(
+        parallel(
+            p_fun(
+                mri_rr, sr.copy(), np.ascontiguousarray(mri_Q), np.array(sol)
+            )  # copy and contig
+            for sr, sol in zip(nas(bem_rr, n_jobs), nas(solution.T, n_jobs))
+        ),
+        axis=0,
+    )
     # The copy()s above should make it so the whole objects don't need to be
     # pickled...
 
     # Only MEG coils are sensitive to the primary current distribution.
-    if coil_type == 'meg':
+    if coil_type == "meg":
         # Primary current contribution (can be calc. in coil/dipole coords)
-        parallel, p_fun, _ = parallel_func(_do_prim_curr, n_jobs)
-        pcc = np.concatenate(parallel(p_fun(r, coils)
-                                      for r in nas(rr, n_jobs)), axis=0)
+        parallel, p_fun, n_jobs = parallel_func(_do_prim_curr, n_jobs)
+        pcc = np.concatenate(parallel(p_fun(r, coils) for r in nas(rr, n_jobs)), axis=0)
         B += pcc
         B *= _MAG_FACTOR
     return B
@@ -478,8 +458,9 @@ def _do_prim_curr(rr, coils):
         pp = _bem_inf_fields(rr[start:stop], rmags, cosmags)
         pp *= ws
         pp.shape = (3 * (stop - start), -1)
-        pc[3 * start:3 * stop] = [bincount(bins, this_pp, bins[-1] + 1)
-                                  for this_pp in pp]
+        pc[3 * start : 3 * stop] = [
+            bincount(bins, this_pp, bins[-1] + 1) for this_pp in pp
+        ]
     return pc
 
 
@@ -520,20 +501,21 @@ def _do_inf_pots(mri_rr, bem_rr, mri_Q, sol):
         # v0 in Hämäläinen et al., 1989 == v_inf in Mosher, et al., 1999
         v0s = _bem_inf_pots(mri_rr[start:stop], bem_rr, mri_Q)
         v0s = v0s.reshape(-1, v0s.shape[2])
-        B[3 * start:3 * stop] = np.dot(v0s, sol)
+        B[3 * start : 3 * stop] = np.dot(v0s, sol)
     return B
 
 
 # #############################################################################
 # SPHERE COMPUTATION
 
-def _sphere_pot_or_field(rr, mri_rr, mri_Q, coils, sphere, bem_rr,
-                         n_jobs, coil_type):
+
+def _sphere_pot_or_field(rr, mri_rr, mri_Q, coils, solution, bem_rr, n_jobs, coil_type):
     """Do potential or field for spherical model."""
-    fun = _eeg_spherepot_coil if coil_type == 'eeg' else _sphere_field
-    parallel, p_fun, _ = parallel_func(fun, n_jobs)
-    B = np.concatenate(parallel(p_fun(r, coils, sphere)
-                                for r in np.array_split(rr, n_jobs)))
+    fun = _eeg_spherepot_coil if coil_type == "eeg" else _sphere_field
+    parallel, p_fun, n_jobs = parallel_func(fun, n_jobs, max_jobs=len(rr))
+    B = np.concatenate(
+        parallel(p_fun(r, coils, sphere=solution) for r in np.array_split(rr, n_jobs))
+    )
     return B
 
 
@@ -547,7 +529,7 @@ def _sphere_field(rrs, coils, sphere):
     by Matti Hämäläinen, February 1990
     """
     rmags, cosmags, ws, bins = _triage_coils(coils)
-    return _do_sphere_field(rrs, rmags, cosmags, ws, bins, sphere['r0'])
+    return _do_sphere_field(rrs, rmags, cosmags, ws, bins, sphere["r0"])
 
 
 @jit()
@@ -583,8 +565,9 @@ def _do_sphere_field(rrs, rmags, cosmags, ws, bins, r0):
         _jit_cross(v1, rr_, cosmags)
         v2 = np.empty((cosmags.shape[0], 3))
         _jit_cross(v2, rr_, this_poss)
-        xx = ((good * ws).reshape(-1, 1) *
-              (v1 / F.reshape(-1, 1) + v2 * g.reshape(-1, 1)))
+        xx = (good * ws).reshape(-1, 1) * (
+            v1 / F.reshape(-1, 1) + v2 * g.reshape(-1, 1)
+        )
         for jj in range(3):
             zz = bincount(bins, xx[:, jj], n_coils)
             B[3 * ri + jj, :] = zz
@@ -599,24 +582,24 @@ def _eeg_spherepot_coil(rrs, coils, sphere):
     del coils
 
     # Shift to the sphere model coordinates
-    rrs = rrs - sphere['r0']
+    rrs = rrs - sphere["r0"]
 
     B = np.zeros((3 * len(rrs), n_coils))
     for ri, rr in enumerate(rrs):
         # Only process dipoles inside the innermost sphere
-        if np.sqrt(np.dot(rr, rr)) >= sphere['layers'][0]['rad']:
+        if np.sqrt(np.dot(rr, rr)) >= sphere["layers"][0]["rad"]:
             continue
         # fwd_eeg_spherepot_vec
         vval_one = np.zeros((len(rmags), 3))
 
         # Make a weighted sum over the equivalence parameters
-        for eq in range(sphere['nfit']):
+        for eq in range(sphere["nfit"]):
             # Scale the dipole position
-            rd = sphere['mu'][eq] * rr
+            rd = sphere["mu"][eq] * rr
             rd2 = np.sum(rd * rd)
             rd2_inv = 1.0 / rd2
             # Go over all electrodes
-            this_pos = rmags - sphere['r0']
+            this_pos = rmags - sphere["r0"]
 
             # Scale location onto the surface of the sphere (not used)
             # if sphere['scale_pos']:
@@ -642,17 +625,19 @@ def _eeg_spherepot_coil(rrs, coils, sphere):
             c2 = a3 + (a + r) / (r * F)
 
             # Mix them together and scale by lambda/(rd*rd)
-            m1 = (c1 - c2 * rrd)
+            m1 = c1 - c2 * rrd
             m2 = c2 * rd2
 
-            vval_one += (sphere['lambda'][eq] * rd2_inv *
-                         (m1[:, np.newaxis] * rd +
-                          m2[:, np.newaxis] * this_pos))
+            vval_one += (
+                sphere["lambda"][eq]
+                * rd2_inv
+                * (m1[:, np.newaxis] * rd + m2[:, np.newaxis] * this_pos)
+            )
 
             # compute total result
             xx = vval_one * ws[:, np.newaxis]
             zz = np.array([bincount(bins, x, bins[-1] + 1) for x in xx.T])
-            B[3 * ri:3 * ri + 3, :] = zz
+            B[3 * ri : 3 * ri + 3, :] = zz
     # finishing by scaling by 1/(4*M_PI)
     B *= 0.25 / np.pi
     return B
@@ -668,14 +653,14 @@ def _triage_coils(coils):
 _MIN_DIST_LIMIT = 1e-5
 
 
-def _magnetic_dipole_field_vec(rrs, coils, too_close='raise'):
+def _magnetic_dipole_field_vec(rrs, coils, too_close="raise"):
     rmags, cosmags, ws, bins = _triage_coils(coils)
     fwd, min_dist = _compute_mdfv(rrs, rmags, cosmags, ws, bins, too_close)
     if min_dist < _MIN_DIST_LIMIT:
-        msg = 'Coil too close (dist = %g mm)' % (min_dist * 1000,)
-        if too_close == 'raise':
+        msg = f"Coil too close (dist = {min_dist * 1000:g} mm)"
+        if too_close == "raise":
             raise RuntimeError(msg)
-        func = warn if too_close == 'warning' else logger.info
+        func = warn if too_close == "warning" else logger.info
         func(msg)
     return fwd
 
@@ -708,7 +693,7 @@ def _compute_mdfv(rrs, rmags, cosmags, ws, bins, too_close):
         dist2 = dist2_.reshape(-1, 1)
         dist = np.sqrt(dist2)
         min_dist = min(dist.min(), min_dist)
-        if min_dist < _MIN_DIST_LIMIT and too_close == 'raise':
+        if min_dist < _MIN_DIST_LIMIT and too_close == "raise":
             break
         t_ = np.sum(diff * cosmags, axis=1)
         t = t_.reshape(-1, 1)
@@ -722,8 +707,9 @@ def _compute_mdfv(rrs, rmags, cosmags, ws, bins, too_close):
 # #############################################################################
 # MAIN TRIAGING FUNCTION
 
+
 @verbose
-def _prep_field_computation(rr, bem, fwd_data, n_jobs, verbose=None):
+def _prep_field_computation(rr, *, sensors, bem, n_jobs, verbose=None):
     """Precompute and store some things that are used for both MEG and EEG.
 
     Calculation includes multiplication factors, coordinate transforms,
@@ -736,73 +722,54 @@ def _prep_field_computation(rr, bem, fwd_data, n_jobs, verbose=None):
     bem : instance of ConductorModel
         Boundary Element Model information
     fwd_data : dict
-        Dict containing sensor information. Gets updated here with BEM and
-        sensor information for later forward calculations
+        Dict containing sensor information in the head coordinate frame.
+        Gets updated here with BEM and sensor information for later forward
+        calculations.
     %(n_jobs)s
     %(verbose)s
     """
     bem_rr = mults = mri_Q = head_mri_t = None
-    if not bem['is_sphere']:
-        if bem['bem_method'] != FWD.BEM_LINEAR_COLL:
-            raise RuntimeError('only linear collocation supported')
+    if not bem["is_sphere"]:
+        if bem["bem_method"] != FIFF.FIFFV_BEM_APPROX_LINEAR:
+            raise RuntimeError("only linear collocation supported")
         # Store (and apply soon) μ_0/(4π) factor before source computations
-        mults = np.repeat(bem['source_mult'] / (4.0 * np.pi),
-                          [len(s['rr']) for s in bem['surfs']])[np.newaxis, :]
+        mults = np.repeat(
+            bem["source_mult"] / (4.0 * np.pi), [len(s["rr"]) for s in bem["surfs"]]
+        )[np.newaxis, :]
         # Get positions of BEM points for every surface
-        bem_rr = np.concatenate([s['rr'] for s in bem['surfs']])
+        bem_rr = np.concatenate([s["rr"] for s in bem["surfs"]])
 
         # The dipole location and orientation must be transformed
-        head_mri_t = bem['head_mri_t']
-        mri_Q = bem['head_mri_t']['trans'][:3, :3].T
+        head_mri_t = bem["head_mri_t"]
+        mri_Q = bem["head_mri_t"]["trans"][:3, :3].T
 
-    # Compute solution and compensation for dif sensor types ('meg', 'eeg')
-    if len(set(fwd_data['coil_types'])) != len(fwd_data['coil_types']):
-        raise RuntimeError('Non-unique sensor types found')
-    compensators, solutions, csolutions = [], [], []
-    coils_list, ccoils_list = [], []
-    for coil_type, coils, ccoils, info in zip(fwd_data['coil_types'],
-                                              fwd_data['coils_list'],
-                                              fwd_data['ccoils_list'],
-                                              fwd_data['infos']):
-        compensator = solution = csolution = None
-        if len(coils) > 0:  # Only proceed if sensors exist
-            if coil_type == 'meg':
-                # Compose a compensation data set if necessary
-                compensator = _make_ctf_comp_coils(info, coils)
-
-            if not bem['is_sphere']:
-                if coil_type == 'meg':
-                    # MEG field computation matrices for BEM
-                    start = 'Composing the field computation matrix'
-                    logger.info('\n' + start + '...')
-                    cf = FIFF.FIFFV_COORD_HEAD
-                    # multiply solution by "mults" here for simplicity
-                    solution = _bem_specify_coils(bem, coils, cf, mults,
-                                                  n_jobs)
-                    if compensator is not None:
-                        logger.info(start + ' (compensation coils)...')
-                        csolution = _bem_specify_coils(bem, ccoils, cf,
-                                                       mults, n_jobs)
-                else:
-                    # Compute solution for EEG sensor
-                    logger.info('Setting up for EEG...')
-                    solution = _bem_specify_els(bem, coils, mults)
+    solutions = dict()
+    for coil_type in sensors:
+        coils = sensors[coil_type]["defs"]
+        if not bem["is_sphere"]:
+            if coil_type == "meg":
+                # MEG field computation matrices for BEM
+                start = "Composing the field computation matrix"
+                logger.info("\n" + start + "...")
+                cf = FIFF.FIFFV_COORD_HEAD
+                # multiply solution by "mults" here for simplicity
+                solution = _bem_specify_coils(bem, coils, cf, mults, n_jobs)
             else:
-                solution = csolution = bem
-                if coil_type == 'eeg':
-                    logger.info('Using the equivalent source approach in the '
-                                'homogeneous sphere for EEG')
-            coils = _triage_coils(coils)
-            if ccoils is not None and len(ccoils) > 0:
-                ccoils = _triage_coils(ccoils)
-        coils_list.append(coils)
-        ccoils_list.append(ccoils)
-        compensators.append(compensator)
-        solutions.append(solution)
-        csolutions.append(csolution)
+                # Compute solution for EEG sensor
+                logger.info("Setting up for EEG...")
+                solution = _bem_specify_els(bem, coils, mults)
+        else:
+            solution = bem
+            if coil_type == "eeg":
+                logger.info(
+                    "Using the equivalent source approach in the "
+                    "homogeneous sphere for EEG"
+                )
+        sensors[coil_type]["defs"] = _triage_coils(coils)
+        solutions[coil_type] = solution
 
     # Get appropriate forward physics function depending on sphere or BEM model
-    fun = _sphere_pot_or_field if bem['is_sphere'] else _bem_pot_or_field
+    fun = _sphere_pot_or_field if bem["is_sphere"] else _bem_pot_or_field
 
     # Update fwd_data with
     #    bem_rr (3D BEM vertex positions)
@@ -811,111 +778,125 @@ def _prep_field_computation(rr, bem, fwd_data, n_jobs, verbose=None):
     #    fun (_bem_pot_or_field if not 'sphere'; otherwise _sph_pot_or_field)
     #    solutions (len 2 list; [ndarray, shape (n_MEG_sens, n BEM vertices),
     #                            ndarray, shape (n_EEG_sens, n BEM vertices)]
-    #    csolutions (compensation for solution)
-    fwd_data.update(dict(bem_rr=bem_rr, mri_Q=mri_Q, head_mri_t=head_mri_t,
-                         compensators=compensators, solutions=solutions,
-                         csolutions=csolutions, fun=fun,
-                         coils_list=coils_list, ccoils_list=ccoils_list))
+    fwd_data = dict(
+        bem_rr=bem_rr, mri_Q=mri_Q, head_mri_t=head_mri_t, fun=fun, solutions=solutions
+    )
+    return fwd_data
 
 
 @fill_doc
-def _compute_forwards_meeg(rr, fd, n_jobs, silent=False):
-    """Compute MEG and EEG forward solutions for all sensor types.
-
-    Parameters
-    ----------
-    rr : ndarray, shape (n_dipoles, 3)
-        3D dipole positions in head coordinates
-    fd : dict
-        Dict containing forward data after update in _prep_field_computation
-    %(n_jobs)s
-    silent : bool
-        If True, don't emit logger.info.
-        This saves time over ``verbose`` when this function is called a lot.
-
-    Returns
-    -------
-    Bs : list
-        Each element contains ndarray, shape (3 * n_dipoles, n_sensors) where
-        n_sensors depends on which channel types are requested (MEG and/or EEG)
-    """
-    n_jobs = max(min(n_jobs, len(rr)), 1)
-    Bs = list()
+def _compute_forwards_meeg(rr, *, sensors, fwd_data, n_jobs, silent=False):
+    """Compute MEG and EEG forward solutions for all sensor types."""
+    Bs = dict()
     # The dipole location and orientation must be transformed to mri coords
     mri_rr = None
-    if fd['head_mri_t'] is not None:
-        mri_rr = np.ascontiguousarray(
-            apply_trans(fd['head_mri_t']['trans'], rr))
-    mri_Q, bem_rr, fun = fd['mri_Q'], fd['bem_rr'], fd['fun']
-    for ci in range(len(fd['coils_list'])):
-        coils, ccoils = fd['coils_list'][ci], fd['ccoils_list'][ci]
-        if len(coils) == 0:  # nothing to do
-            Bs.append(np.zeros((3 * len(rr), 0)))
-            continue
-
-        coil_type, compensator = fd['coil_types'][ci], fd['compensators'][ci]
-        solution, csolution = fd['solutions'][ci], fd['csolutions'][ci]
-        info = fd['infos'][ci]
+    if fwd_data["head_mri_t"] is not None:
+        mri_rr = np.ascontiguousarray(apply_trans(fwd_data["head_mri_t"]["trans"], rr))
+    mri_Q, bem_rr, fun = fwd_data["mri_Q"], fwd_data["bem_rr"], fwd_data["fun"]
+    solutions = fwd_data["solutions"]
+    del fwd_data
+    rr = np.ascontiguousarray(rr)  # usually true but not guaranteed, e.g. in dipole.py
+    for coil_type, sens in sensors.items():
+        coils = sens["defs"]
+        compensator = sens.get("compensator", None)
+        post_picks = sens.get("post_picks", None)
+        solution = solutions.get(coil_type, None)
 
         # Do the actual forward calculation for a list MEG/EEG sensors
         if not silent:
-            logger.info('Computing %s at %d source location%s '
-                        '(free orientations)...'
-                        % (coil_type.upper(), len(rr), _pl(rr)))
+            logger.info(
+                f"Computing {coil_type.upper()} at {len(rr)} source location{_pl(rr)} "
+                "(free orientations)..."
+            )
         # Calculate forward solution using spherical or BEM model
-        B = fun(rr, mri_rr, mri_Q, coils, solution, bem_rr, n_jobs,
-                coil_type)
+        B = fun(
+            rr,
+            mri_rr,
+            mri_Q,
+            coils=coils,
+            solution=solution,
+            bem_rr=bem_rr,
+            n_jobs=n_jobs,
+            coil_type=coil_type,
+        )
 
         # Compensate if needed (only done for MEG systems w/compensation)
         if compensator is not None:
-            # Compute the field in the compensation sensors
-            work = fun(rr, mri_rr, mri_Q, ccoils, csolution, bem_rr,
-                       n_jobs, coil_type)
-            # Combine solutions so we can do the compensation
-            both = np.zeros((work.shape[0], B.shape[1] + work.shape[1]))
-            picks = pick_types(info, meg=True, ref_meg=False, exclude=[])
-            both[:, picks] = B
-            picks = pick_types(info, meg=False, ref_meg=True, exclude=[])
-            both[:, picks] = work
-            B = np.dot(both, compensator.T)
-        Bs.append(B)
+            B = B @ compensator.T
+        if post_picks is not None:
+            B = B[:, post_picks]
+        Bs[coil_type] = B
     return Bs
 
 
 @verbose
-def _compute_forwards(rr, bem, coils_list, ccoils_list, infos, coil_types,
-                      n_jobs, verbose=None):
-    """Compute the MEG and EEG forward solutions.
-
-    This effectively combines compute_forward_meg and compute_forward_eeg
-    from MNE-C.
-
-    Parameters
-    ----------
-    rr : ndarray, shape (n_sources, 3)
-        3D dipole in head coordinates
-    bem : instance of ConductorModel
-        Boundary Element Model information for all surfaces
-    coils_list : list
-        List of MEG and/or EEG sensor information dicts
-    ccoils_list : list
-        Optional list of MEG compensation information
-    coil_types : list of str
-        Sensor types. May contain 'meg' and/or 'eeg'
-    %(n_jobs)s
-    infos : list, len(2)
-        infos[0] is MEG info, infos[1] is EEG info
-
-    Returns
-    -------
-    Bs : list of ndarray
-        Each element contains ndarray, shape (3 * n_dipoles, n_sensors) where
-        n_sensors depends on which channel types are requested (MEG and/or EEG)
-    """
+def _compute_forwards(rr, *, bem, sensors, n_jobs, verbose=None):
+    """Compute the MEG and EEG forward solutions."""
     # Split calculation into two steps to save (potentially) a lot of time
     # when e.g. dipole fitting
-    fwd_data = dict(coils_list=coils_list, ccoils_list=ccoils_list,
-                    infos=infos, coil_types=coil_types)
-    _prep_field_computation(rr, bem, fwd_data, n_jobs)
-    Bs = _compute_forwards_meeg(rr, fwd_data, n_jobs)
+    solver = bem.get("solver", "mne")
+    _check_option("solver", solver, ("mne", "openmeeg"))
+    if bem["is_sphere"] or solver == "mne":
+        # This modifies "sensors" in place, so let's copy it in case the calling
+        # function needs to reuse it (e.g., in simulate_raw.py)
+        sensors = deepcopy(sensors)
+        fwd_data = _prep_field_computation(rr, sensors=sensors, bem=bem, n_jobs=n_jobs)
+        Bs = _compute_forwards_meeg(
+            rr, sensors=sensors, fwd_data=fwd_data, n_jobs=n_jobs
+        )
+    else:
+        Bs = _compute_forwards_openmeeg(rr, bem=bem, sensors=sensors)
+    n_sensors_want = sum(len(s["ch_names"]) for s in sensors.values())
+    n_sensors = sum(B.shape[1] for B in Bs.values())
+    n_sources = list(Bs.values())[0].shape[0]
+    assert (n_sources, n_sensors) == (len(rr) * 3, n_sensors_want)
+    return Bs
+
+
+def _compute_forwards_openmeeg(rr, *, bem, sensors):
+    """Compute the MEG and EEG forward solutions for OpenMEEG."""
+    if len(bem["surfs"]) != 3:
+        raise RuntimeError("Only 3-layer BEM is supported for OpenMEEG.")
+    om = _import_openmeeg("compute a forward solution using OpenMEEG")
+    hminv = om.SymMatrix(bem["solution"])
+    geom = _make_openmeeg_geometry(bem, invert_transform(bem["head_mri_t"]))
+
+    # Make dipoles for all XYZ orientations
+    dipoles = np.c_[
+        np.kron(rr.T, np.ones(3)[None, :]).T,
+        np.kron(np.ones(len(rr))[:, None], np.eye(3)),
+    ]
+    dipoles = np.asfortranarray(dipoles)
+    dipoles = om.Matrix(dipoles)
+    dsm = om.DipSourceMat(geom, dipoles, "Brain")
+    Bs = dict()
+    if "eeg" in sensors:
+        rmags, _, ws, bins = _concatenate_coils(sensors["eeg"]["defs"])
+        rmags = np.asfortranarray(rmags.astype(np.float64))
+        eeg_sensors = om.Sensors(om.Matrix(np.asfortranarray(rmags)), geom)
+        h2em = om.Head2EEGMat(geom, eeg_sensors)
+        eeg_fwd_full = om.GainEEG(hminv, dsm, h2em).array()
+        Bs["eeg"] = np.array(
+            [bincount(bins, ws * x, bins[-1] + 1) for x in eeg_fwd_full.T], float
+        )
+    if "meg" in sensors:
+        rmags, cosmags, ws, bins = _concatenate_coils(sensors["meg"]["defs"])
+        rmags = np.asfortranarray(rmags.astype(np.float64))
+        cosmags = np.asfortranarray(cosmags.astype(np.float64))
+        labels = [str(ii) for ii in range(len(rmags))]
+        weights = radii = np.ones(len(labels))
+        meg_sensors = om.Sensors(labels, rmags, cosmags, weights, radii)
+        h2mm = om.Head2MEGMat(geom, meg_sensors)
+        ds2mm = om.DipSource2MEGMat(dipoles, meg_sensors)
+        meg_fwd_full = om.GainMEG(hminv, dsm, h2mm, ds2mm).array()
+        B = np.array(
+            [bincount(bins, ws * x, bins[-1] + 1) for x in meg_fwd_full.T], float
+        )
+        compensator = sensors["meg"].get("compensator", None)
+        post_picks = sensors["meg"].get("post_picks", None)
+        if compensator is not None:
+            B = B @ compensator.T
+        if post_picks is not None:
+            B = B[:, post_picks]
+        Bs["meg"] = B
     return Bs
