@@ -1,49 +1,51 @@
-# -*- coding: utf-8 -*-
 """Some utility functions."""
-# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#
+
+# Authors: The MNE-Python contributors.
 # License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
 
 import contextlib
 import importlib
 import inspect
-from io import StringIO
-import re
-import sys
 import logging
 import os.path as op
+import re
+import sys
 import warnings
-from typing import Any, Callable, TypeVar
+from collections.abc import Callable
+from io import StringIO
+from typing import Any, TypeVar
+
+from decorator import FunctionMaker
 
 from .docs import fill_doc
-from ..externals.decorator import FunctionMaker
 
-
-logger = logging.getLogger('mne')  # one selection here used across mne-python
+logger = logging.getLogger("mne")  # one selection here used across mne-python
 logger.propagate = False  # don't propagate (in case of multiple imports)
 
 
 # class to provide frame information (should be low overhead, just on logger
 # calls)
 
+
 class _FrameFilter(logging.Filter):
     def __init__(self):
         self.add_frames = 0
 
     def filter(self, record):
-        record.frame_info = 'Unknown'
+        record.frame_info = "Unknown"
         if self.add_frames:
             # 5 is the offset necessary to get out of here and the logging
             # module, reversal is to put the oldest at the top
             frame_info = _frame_info(5 + self.add_frames)[5:][::-1]
             if len(frame_info):
-                frame_info[-1] = (frame_info[-1] + ' :').ljust(30)
+                frame_info[-1] = (frame_info[-1] + " :").ljust(30)
                 if len(frame_info) > 1:
-                    frame_info[0] = '┌' + frame_info[0]
-                    frame_info[-1] = '└' + frame_info[-1]
+                    frame_info[0] = "┌" + frame_info[0]
+                    frame_info[-1] = "└" + frame_info[-1]
                 for ii, info in enumerate(frame_info[1:-1], 1):
-                    frame_info[ii] = '├' + info
-                record.frame_info = '\n'.join(frame_info)
+                    frame_info[ii] = "├" + info
+                record.frame_info = "\n".join(frame_info)
         return True
 
 
@@ -53,7 +55,7 @@ logger.addFilter(_filter)
 
 # Provide help for static type checkers:
 # https://mypy.readthedocs.io/en/stable/generics.html#declaring-decorators
-_FuncT = TypeVar('_FuncT', bound=Callable[..., Any])
+_FuncT = TypeVar("_FuncT", bound=Callable[..., Any])
 
 
 def verbose(function: _FuncT) -> _FuncT:
@@ -102,65 +104,87 @@ def verbose(function: _FuncT) -> _FuncT:
     except TypeError:  # nothing to add
         pass
 
-    # Anything using verbose should either have `verbose=None` in the signature
-    # or have a `self.verbose` attribute (if in a method). This code path
-    # will raise an error if neither is the case.
+    # Anything using verbose should have `verbose=None` in the signature.
+    # This code path will raise an error if this is not the case.
     body = """\
 def %(name)s(%(signature)s):\n
     try:
-        verbose
-    except UnboundLocalError:
-        try:
-            verbose = self.verbose
-        except NameError:
-            raise RuntimeError('Function %%s does not accept verbose parameter'
-                               %% (_function_,))
-        except AttributeError:
-            raise RuntimeError('Method %%s class does not have self.verbose'
-                               %% (_function_,))
-    else:
-        if verbose is None:
-            try:
-                verbose = self.verbose
-            except (NameError, AttributeError):
-                pass
-    if verbose is not None:
+        do_level_change = verbose is not None
+    except (NameError, UnboundLocalError):
+        raise RuntimeError('Function/method %%s does not accept verbose '
+                           'parameter' %% (_function_,)) from None
+    if do_level_change:
         with _use_log_level_(verbose):
             return _function_(%(shortsignature)s)
     else:
         return _function_(%(shortsignature)s)"""
-    evaldict = dict(
-        _use_log_level_=use_log_level, _function_=function)
-    fm = FunctionMaker(function, None, None, None, None, function.__module__)
-    attrs = dict(__wrapped__=function, __qualname__=function.__qualname__,
-                 __globals__=function.__globals__)
+    evaldict = dict(_use_log_level_=use_log_level, _function_=function)
+    fm = FunctionMaker(function)
+    attrs = dict(
+        __wrapped__=function,
+        __qualname__=function.__qualname__,
+        __globals__=function.__globals__,
+    )
     return fm.make(body, evaldict, addsource=True, **attrs)
 
 
-class use_log_level(object):
-    """Context handler for logging level.
+@fill_doc
+class use_log_level:
+    """Context manager for logging level.
 
     Parameters
     ----------
-    level : int
-        The level to use.
-    add_frames : int | None
-        Number of stack frames to include.
+    %(verbose)s
+    %(add_frames)s
+
+    See Also
+    --------
+    mne.verbose
+
+    Notes
+    -----
+    See the :ref:`logging documentation <tut-logging>` for details.
+
+    Examples
+    --------
+    >>> from mne import use_log_level
+    >>> from mne.utils import logger
+    >>> with use_log_level(False):
+    ...     # Most MNE logger messages are "info" level, False makes them not
+    ...     # print:
+    ...     logger.info('This message will not be printed')
+    >>> with use_log_level(True):
+    ...     # Using verbose=True in functions, methods, or this context manager
+    ...     # will ensure they are printed
+    ...     logger.info('This message will be printed!')
+    This message will be printed!
     """
 
-    def __init__(self, level, add_frames=None):  # noqa: D102
-        self.level = level
-        self.add_frames = add_frames
-        self.old_frames = _filter.add_frames
+    def __init__(self, verbose=None, *, add_frames=None):
+        self._level = verbose
+        self._add_frames = add_frames
+        self._old_frames = _filter.add_frames
 
     def __enter__(self):  # noqa: D105
-        self.old_level = set_log_level(self.level, True, self.add_frames)
+        self._old_level = set_log_level(
+            self._level, return_old_level=True, add_frames=self._add_frames
+        )
 
     def __exit__(self, *args):  # noqa: D105
-        add_frames = self.old_frames if self.add_frames is not None else None
-        set_log_level(self.old_level, add_frames=add_frames)
+        add_frames = self._old_frames if self._add_frames is not None else None
+        set_log_level(self._old_level, add_frames=add_frames)
 
 
+_LOGGING_TYPES = dict(
+    DEBUG=logging.DEBUG,
+    INFO=logging.INFO,
+    WARNING=logging.WARNING,
+    ERROR=logging.ERROR,
+    CRITICAL=logging.CRITICAL,
+)
+
+
+@fill_doc
 def set_log_level(verbose=None, return_old_level=False, add_frames=None):
     """Set the logging level.
 
@@ -175,54 +199,56 @@ def set_log_level(verbose=None, return_old_level=False, add_frames=None):
         it doesn't exist, defaults to INFO.
     return_old_level : bool
         If True, return the old verbosity level.
-    add_frames : int | None
-        If int, enable (>=1) or disable (0) the printing of stack frame
-        information using formatting. Default (None) does not change the
-        formatting. This can add overhead so is meant only for debugging.
+    %(add_frames)s
 
     Returns
     -------
     old_level : int
         The old level. Only returned if ``return_old_level`` is True.
     """
-    from .config import get_config
-    from .check import _check_option, _validate_type
-    _validate_type(verbose, (bool, str, int, None), 'verbose')
-    if verbose is None:
-        verbose = get_config('MNE_LOGGING_LEVEL', 'INFO')
-    elif isinstance(verbose, bool):
-        if verbose is True:
-            verbose = 'INFO'
-        else:
-            verbose = 'WARNING'
-    if isinstance(verbose, str):
-        verbose = verbose.upper()
-        logging_types = dict(DEBUG=logging.DEBUG, INFO=logging.INFO,
-                             WARNING=logging.WARNING, ERROR=logging.ERROR,
-                             CRITICAL=logging.CRITICAL)
-        _check_option('verbose', verbose, logging_types, '(when a string)')
-        verbose = logging_types[verbose]
     old_verbose = logger.level
+    verbose = _parse_verbose(verbose)
+
     if verbose != old_verbose:
         logger.setLevel(verbose)
     if add_frames is not None:
         _filter.add_frames = int(add_frames)
-        fmt = '%(frame_info)s ' if add_frames else ''
-        fmt += '%(message)s'
+        fmt = "%(frame_info)s " if add_frames else ""
+        fmt += "%(message)s"
         fmt = logging.Formatter(fmt)
         for handler in logger.handlers:
             handler.setFormatter(fmt)
-    return (old_verbose if return_old_level else None)
+    return old_verbose if return_old_level else None
 
 
-def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
+def _parse_verbose(verbose):
+    from .check import _check_option, _validate_type
+    from .config import get_config
+
+    _validate_type(verbose, (bool, str, int, None), "verbose")
+    if verbose is None:
+        verbose = get_config("MNE_LOGGING_LEVEL", "INFO")
+    elif isinstance(verbose, bool):
+        if verbose is True:
+            verbose = "INFO"
+        else:
+            verbose = "WARNING"
+    if isinstance(verbose, str):
+        verbose = verbose.upper()
+        _check_option("verbose", verbose, _LOGGING_TYPES, "(when a string)")
+        verbose = _LOGGING_TYPES[verbose]
+
+    return verbose
+
+
+def set_log_file(fname=None, output_format="%(message)s", overwrite=None):
     """Set the log to print to a file.
 
     Parameters
     ----------
-    fname : str, or None
+    fname : path-like | None
         Filename of the log to print to. If None, stdout is used.
-        To suppress log outputs, use set_log_level('WARN').
+        To suppress log outputs, use set_log_level('WARNING').
     output_format : str
         Format of the output messages. See the following for examples:
 
@@ -240,16 +266,20 @@ def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
         if op.isfile(fname) and overwrite is None:
             # Don't use warn() here because we just want to
             # emit a warnings.warn here (not logger.warn)
-            warnings.warn('Log entries will be appended to the file. Use '
-                          'overwrite=False to avoid this message in the '
-                          'future.', RuntimeWarning, stacklevel=2)
+            warnings.warn(
+                "Log entries will be appended to the file. Use "
+                "overwrite=False to avoid this message in the "
+                "future.",
+                RuntimeWarning,
+                stacklevel=2,
+            )
             overwrite = False
-        mode = 'w' if overwrite else 'a'
+        mode = "w" if overwrite else "a"
         lh = logging.FileHandler(fname, mode=mode)
     else:
-        """ we should just be able to do:
-                lh = logging.StreamHandler(sys.stdout)
-            but because doctests uses some magic on stdout, we have to do this:
+        """we should just be able to do:
+            lh = logging.StreamHandler(sys.stdout)
+        but because doctests uses some magic on stdout, we have to do this:
         """
         lh = logging.StreamHandler(WrapStdOut())
 
@@ -261,7 +291,7 @@ def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
 def _remove_close_handlers(logger):
     for h in list(logger.handlers):
         # only remove our handlers (get along nicely with nose)
-        if isinstance(h, (logging.FileHandler, logging.StreamHandler)):
+        if isinstance(h, logging.FileHandler | logging.StreamHandler):
             if isinstance(h, logging.FileHandler):
                 h.close()
             logger.removeHandler(h)
@@ -278,28 +308,46 @@ class ClosingStringIO(StringIO):
         return out
 
 
-class catch_logging(object):
+class catch_logging:
     """Store logging.
 
     This will remove all other logging handlers, and return the handler to
     stdout when complete.
     """
 
+    def __init__(self, verbose=None):
+        self.verbose = verbose
+
     def __enter__(self):  # noqa: D105
+        if self.verbose is not None:
+            self._ctx = use_log_level(self.verbose)
+        else:
+            self._ctx = contextlib.nullcontext()
         self._data = ClosingStringIO()
         self._lh = logging.StreamHandler(self._data)
-        self._lh.setFormatter(logging.Formatter('%(message)s'))
+        self._lh.setFormatter(logging.Formatter("%(message)s"))
         self._lh._mne_file_like = True  # monkey patch for warn() use
         _remove_close_handlers(logger)
         logger.addHandler(self._lh)
+        self._ctx.__enter__()
         return self._data
 
     def __exit__(self, *args):  # noqa: D105
+        self._ctx.__exit__(*args)
         logger.removeHandler(self._lh)
         set_log_file(None)
 
 
-class WrapStdOut(object):
+@contextlib.contextmanager
+def _record_warnings():
+    # this is a helper that mostly acts like pytest.warns(None) did before
+    # pytest 7
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        yield w
+
+
+class WrapStdOut:
     """Dynamically wrap to sys.stdout.
 
     This makes packages that monkey-patch sys.stdout (e.g.doctest,
@@ -312,14 +360,13 @@ class WrapStdOut(object):
         if hasattr(sys.stdout, name):
             return getattr(sys.stdout, name)
         else:
-            raise AttributeError("'file' object has not attribute '%s'" % name)
+            raise AttributeError(f"'file' object has not attribute '{name}'")
 
 
-_verbose_dec_re = re.compile('^<decorator-gen-[0-9]+>$')
+_verbose_dec_re = re.compile("^<decorator-gen-[0-9]+>$")
 
 
-def warn(message, category=RuntimeWarning, module='mne',
-         ignore_namespaces=('mne',)):
+def warn(message, category=RuntimeWarning, module="mne", ignore_namespaces=("mne",)):
     """Emit a warning with trace outside the mne namespace.
 
     This function takes arguments like warnings.warn, and sends messages
@@ -344,7 +391,7 @@ def warn(message, category=RuntimeWarning, module='mne',
     root_dirs = [importlib.import_module(ns) for ns in ignore_namespaces]
     root_dirs = [op.dirname(ns.__file__) for ns in root_dirs]
     frame = None
-    if logger.level <= logging.WARN:
+    if logger.level <= logging.WARNING:
         frame = inspect.currentframe()
         while frame:
             fname = frame.f_code.co_filename
@@ -353,9 +400,13 @@ def warn(message, category=RuntimeWarning, module='mne',
             if not _verbose_dec_re.search(fname):
                 # treat tests as scripts
                 # and don't capture unittest/case.py (assert_raises)
-                if not (any(fname.startswith(rd) for rd in root_dirs) or
-                        ('unittest' in fname and 'case' in fname)) or \
-                        op.basename(op.dirname(fname)) == 'tests':
+                if (
+                    not (
+                        any(fname.startswith(rd) for rd in root_dirs)
+                        or ("unittest" in fname and "case" in fname)
+                    )
+                    or op.basename(op.dirname(fname)) == "tests"
+                ):
                     break
             frame = frame.f_back
         del frame
@@ -363,16 +414,25 @@ def warn(message, category=RuntimeWarning, module='mne',
         # because we move out of the MNE stack, so warnings won't properly
         # recognize the module name (and our warnings.simplefilter will fail)
         warnings.warn_explicit(
-            message, category, fname, lineno, module,
-            globals().get('__warningregistry__', {}))
+            message,
+            category,
+            fname,
+            lineno,
+            module,
+            globals().get("__warningregistry__", {}),
+        )
     # To avoid a duplicate warning print, we only emit the logger.warning if
     # one of the handlers is a FileHandler. See gh-5592
     # But it's also nice to be able to do:
     # with mne.utils.use_log_level('warning', add_frames=3):
     # so also check our add_frames attribute.
-    if any(isinstance(h, logging.FileHandler) or getattr(h, '_mne_file_like',
-                                                         False)
-           for h in logger.handlers) or _filter.add_frames:
+    if (
+        any(
+            isinstance(h, logging.FileHandler) or getattr(h, "_mne_file_like", False)
+            for h in logger.handlers
+        )
+        or _filter.add_frames
+    ):
         logger.warning(message)
 
 
@@ -382,7 +442,7 @@ def _get_call_line():
     if _verbose_dec_re.search(frame.f_code.co_filename):
         frame = frame.f_back
     context = inspect.getframeinfo(frame).code_context
-    context = 'unknown' if context is None else context[0].strip()
+    context = "unknown" if context is None else context[0].strip()
     return context
 
 
@@ -399,57 +459,21 @@ def filter_out_warnings(warn_record, category=None, match=None):
 
     match : str | None
         text or regex that matches the error message to filter out
-
-    Examples
-    --------
-    This can be used as::
-
-        >>> import pytest
-        >>> import warnings
-        >>> from mne.utils import filter_out_warnings
-        >>> with pytest.warns(None) as recwarn:
-        ...     warnings.warn("value must be 0 or None", UserWarning)
-        >>> filter_out_warnings(recwarn, match=".* 0 or None")
-        >>> assert len(recwarn.list) == 0
-
-        >>> with pytest.warns(None) as recwarn:
-        ...     warnings.warn("value must be 42", UserWarning)
-        >>> filter_out_warnings(recwarn, match=r'.* must be \d+$')
-        >>> assert len(recwarn.list) == 0
-
-        >>> with pytest.warns(None) as recwarn:
-        ...     warnings.warn("this is not here", UserWarning)
-        >>> filter_out_warnings(recwarn, match=r'.* must be \d+$')
-        >>> assert len(recwarn.list) == 1
     """
-    regexp = re.compile('.*' if match is None else match)
-    is_category = [w.category == category if category is not None else True
-                   for w in warn_record._list]
-    is_match = [regexp.match(w.message.args[0]) is not None
-                for w in warn_record._list]
-    ind = [ind for ind, (c, m) in enumerate(zip(is_category, is_match))
-           if c and m]
+    regexp = re.compile(".*" if match is None else match)
+    is_category = [
+        w.category == category if category is not None else True
+        for w in warn_record._list
+    ]
+    is_match = [regexp.match(w.message.args[0]) is not None for w in warn_record._list]
+    ind = [ind for ind, (c, m) in enumerate(zip(is_category, is_match)) if c and m]
 
     for i in reversed(ind):
         warn_record._list.pop(i)
 
 
-class ETSContext(object):
-    """Add more meaningful message to errors generated by ETS Toolkit."""
-
-    def __enter__(self):  # noqa: D105
-        pass
-
-    def __exit__(self, type, value, traceback):  # noqa: D105
-        if isinstance(value, SystemExit) and value.code.\
-                startswith("This program needs access to the screen"):
-            value.code += ("\nThis can probably be solved by setting "
-                           "ETS_TOOLKIT=qt4. On bash, type\n\n    $ export "
-                           "ETS_TOOLKIT=qt4\n\nand run the command again.")
-
-
 @contextlib.contextmanager
-def wrapped_stdout(indent='', cull_newlines=False):
+def wrapped_stdout(indent="", cull_newlines=False):
     """Wrap stdout writes to logger.info, with an optional indent prefix.
 
     Parameters
@@ -467,12 +491,12 @@ def wrapped_stdout(indent='', cull_newlines=False):
     finally:
         sys.stdout = orig_stdout
         pending_newlines = 0
-        for line in my_out.getvalue().split('\n'):
+        for line in my_out.getvalue().split("\n"):
             if not line.strip() and cull_newlines:
                 pending_newlines += 1
                 continue
             for _ in range(pending_newlines):
-                logger.info('\n')
+                logger.info("\n")
             logger.info(indent + line)
 
 
@@ -483,16 +507,21 @@ def _frame_info(n):
         infos = list()
         for _ in range(n):
             try:
-                name = frame.f_globals['__name__']
+                name = frame.f_globals["__name__"]
             except KeyError:  # in our verbose dec
                 pass
             else:
-                infos.append(f'{name.lstrip("mne.")}:{frame.f_lineno}')
+                infos.append(f"{name.lstrip('mne.')}:{frame.f_lineno}")
             frame = frame.f_back
             if frame is None:
                 break
         return infos
     except Exception:
-        return ['unknown']
+        return ["unknown"]
     finally:
         del frame
+
+
+def _verbose_safe_false(*, level="warning"):
+    lev = _LOGGING_TYPES[level.upper()]
+    return lev if logger.level <= lev else None
