@@ -1,22 +1,72 @@
 """ABCs."""
 
-# Authors: Guillaume Favelier <guillaume.favelier@gmail.com
-#          Eric Larson <larson.eric.d@gmail.com>
-#
-# License: Simplified BSD
+# Authors: The MNE-Python contributors.
+# License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
 
-from abc import ABC, abstractmethod, abstractclassmethod
-from contextlib import nullcontext
 import warnings
+from abc import ABC, abstractclassmethod, abstractmethod
 
-from ..utils import tight_layout
+from ..ui_events import TimeChange, publish
+
+
+class Figure3D(ABC):
+    """Class that refers to a 3D figure.
+
+    .. note::
+        This class should not be instantiated directly via
+        ``mne.viz.Figure3D(...)``. Instead, use
+        :func:`mne.viz.create_3d_figure`.
+
+    See Also
+    --------
+    mne.viz.create_3d_figure
+    """
+
+    # Here we use _init rather than __init__ so that users are less tempted to
+    # instantiate the class directly. It also helps us
+    # document the class more easily, as we don't have to say what all the
+    # params are in public docs.
+
+    @abstractclassmethod
+    def _init(
+        self,
+        fig=None,
+        size=(600, 600),
+        bgcolor=(0.0, 0.0, 0.0),
+        *,
+        title="MNE 3D Figure",
+        show=False,
+        shape=(1, 1),
+        splash=False,
+    ):
+        pass
+
+    @property
+    def plotter(self):
+        """The native 3D plotting widget.
+
+        Returns
+        -------
+        plotter : instance of pyvista.Plotter
+            The plotter. Useful for interacting with the native 3D library.
+        """
+        return self._plotter
 
 
 class _AbstractRenderer(ABC):
-
     @abstractclassmethod
-    def __init__(self, fig=None, size=(600, 600), bgcolor=(0., 0., 0.),
-                 name=None, show=False, shape=(1, 1)):
+    def __init__(
+        self,
+        fig=None,
+        size=(600, 600),
+        bgcolor=(0.0, 0.0, 0.0),
+        *,
+        name="MNE-Python 3D Figure",
+        show=False,
+        shape=(1, 1),
+        splash=False,
+    ):
         """Set up the scene."""
         pass
 
@@ -41,11 +91,56 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def mesh(self, x, y, z, triangles, color, opacity=1.0, shading=False,
-             backface_culling=False, scalars=None, colormap=None,
-             vmin=None, vmax=None, interpolate_before_map=True,
-             representation='surface', line_width=1., normals=None,
-             polygon_offset=None, **kwargs):
+    def legend(self, labels, border=False, size=0.1, face="triangle", loc="upper left"):
+        """Add a legend to the scene.
+
+        Parameters
+        ----------
+        labels : list of tuples
+            Each entry must contain two strings, (label, color),
+            where ``label`` is the name of the item to add, and
+            ``color`` is the color of the label to add.
+        border : bool
+            Controls if there will be a border around the legend.
+            The default is False.
+        size : float
+            The size of the entire figure window.
+        loc : str
+            The location of the legend.
+        face : str
+            Face shape of legend face.  One of the following:
+
+            * None: ``None``
+            * Line: ``"-"`` or ``"line"``
+            * Triangle: ``"^"`` or ``'triangle'``
+            * Circle: ``"o"`` or ``'circle'``
+            * Rectangle: ``"r"`` or ``'rectangle'``
+        """
+        pass
+
+    @abstractclassmethod
+    def mesh(
+        self,
+        x,
+        y,
+        z,
+        triangles,
+        color,
+        opacity=1.0,
+        *,
+        backface_culling=False,
+        scalars=None,
+        colormap=None,
+        vmin=None,
+        vmax=None,
+        interpolate_before_map=True,
+        representation="surface",
+        line_width=1.0,
+        normals=None,
+        polygon_offset=None,
+        name=None,
+        **kwargs,
+    ):
         """Add a mesh in the scene.
 
         Parameters
@@ -72,11 +167,11 @@ class _AbstractRenderer(ABC):
             The scalar valued associated to the vertices.
         vmin : float | None
             vmin is used to scale the colormap.
-            If None, the min of the data will be used
+            If None, the min of the data will be used.
         vmax : float | None
             vmax is used to scale the colormap.
-            If None, the max of the data will be used
-        colormap :
+            If None, the max of the data will be used.
+        colormap : str | np.ndarray | matplotlib.colors.Colormap | None
             The colormap to use.
         interpolate_before_map :
             Enabling makes for a smoother scalars display. Default is True.
@@ -90,6 +185,8 @@ class _AbstractRenderer(ABC):
             The array containing the normal of each vertex.
         polygon_offset : float
             If not None, the factor used to resolve coincident topology.
+        name : str | None
+            The name of the mesh.
         kwargs : args
             The arguments to pass to triangular_mesh
 
@@ -101,9 +198,20 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def contour(self, surface, scalars, contours, width=1.0, opacity=1.0,
-                vmin=None, vmax=None, colormap=None,
-                normalized_colormap=False, kind='line', color=None):
+    def contour(
+        self,
+        surface,
+        scalars,
+        contours,
+        width=1.0,
+        opacity=1.0,
+        vmin=None,
+        vmax=None,
+        colormap=None,
+        normalized_colormap=False,
+        kind="line",
+        color=None,
+    ):
         """Add a contour in the scene.
 
         Parameters
@@ -120,17 +228,17 @@ class _AbstractRenderer(ABC):
             The opacity of the contour.
         vmin : float | None
             vmin is used to scale the colormap.
-            If None, the min of the data will be used
+            If None, the min of the data will be used.
         vmax : float | None
             vmax is used to scale the colormap.
-            If None, the max of the data will be used
-        colormap :
+            If None, the max of the data will be used.
+        colormap : str | np.ndarray | matplotlib.colors.Colormap | None
             The colormap to use.
         normalized_colormap : bool
             Specify if the values of the colormap are between 0 and 1.
         kind : 'line' | 'tube'
             The type of the primitives to use to display the contours.
-        color :
+        color : tuple | str
             The color of the mesh as a tuple (red, green, blue) of float
             values between 0 and 1 or a valid color name (i.e. 'white'
             or 'w').
@@ -138,10 +246,21 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def surface(self, surface, color=None, opacity=1.0,
-                vmin=None, vmax=None, colormap=None,
-                normalized_colormap=False, scalars=None,
-                backface_culling=False, polygon_offset=None):
+    def surface(
+        self,
+        surface,
+        color=None,
+        opacity=1.0,
+        vmin=None,
+        vmax=None,
+        colormap=None,
+        normalized_colormap=False,
+        scalars=None,
+        backface_culling=False,
+        polygon_offset=None,
+        *,
+        name=None,
+    ):
         """Add a surface in the scene.
 
         Parameters
@@ -156,11 +275,11 @@ class _AbstractRenderer(ABC):
             The opacity of the surface.
         vmin : float | None
             vmin is used to scale the colormap.
-            If None, the min of the data will be used
+            If None, the min of the data will be used.
         vmax : float | None
             vmax is used to scale the colormap.
-            If None, the max of the data will be used
-        colormap :
+            If None, the max of the data will be used.
+        colormap : str | np.ndarray | matplotlib.colors.Colormap | None
             The colormap to use.
         scalars : ndarray, shape (n_vertices,)
             The scalar valued associated to the vertices.
@@ -168,13 +287,22 @@ class _AbstractRenderer(ABC):
             If True, enable backface culling on the surface.
         polygon_offset : float
             If not None, the factor used to resolve coincident topology.
+        name : str | None
+            Name of the surface.
         """
         pass
 
     @abstractclassmethod
-    def sphere(self, center, color, scale, opacity=1.0,
-               resolution=8, backface_culling=False,
-               radius=None):
+    def sphere(
+        self,
+        center,
+        color,
+        scale,
+        opacity=1.0,
+        resolution=8,
+        backface_culling=False,
+        radius=None,
+    ):
         """Add sphere in the scene.
 
         Parameters
@@ -197,14 +325,24 @@ class _AbstractRenderer(ABC):
             If True, enable backface culling on the sphere(s).
         radius : float | None
             Replace the glyph scaling by a fixed radius value for each
-            sphere (not supported by mayavi).
+            sphere.
         """
         pass
 
     @abstractclassmethod
-    def tube(self, origin, destination, radius=0.001, color='white',
-             scalars=None, vmin=None, vmax=None, colormap='RdBu',
-             normalized_colormap=False, reverse_lut=False):
+    def tube(
+        self,
+        origin,
+        destination,
+        radius=0.001,
+        color="white",
+        scalars=None,
+        vmin=None,
+        vmax=None,
+        colormap="RdBu",
+        normalized_colormap=False,
+        reverse_lut=False,
+    ):
         """Add tube in the scene.
 
         Parameters
@@ -223,11 +361,11 @@ class _AbstractRenderer(ABC):
             The optional scalar data to use.
         vmin : float | None
             vmin is used to scale the colormap.
-            If None, the min of the data will be used
+            If None, the min of the data will be used.
         vmax : float | None
             vmax is used to scale the colormap.
-            If None, the max of the data will be used
-        colormap :
+            If None, the max of the data will be used.
+        colormap : str | np.ndarray | matplotlib.colors.Colormap | None
             The colormap to use.
         opacity : float
             The opacity of the tube(s).
@@ -246,11 +384,31 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def quiver3d(self, x, y, z, u, v, w, color, scale, mode, resolution=8,
-                 glyph_height=None, glyph_center=None, glyph_resolution=None,
-                 opacity=1.0, scale_mode='none', scalars=None,
-                 backface_culling=False, colormap=None, vmin=None, vmax=None,
-                 line_width=2., name=None):
+    def quiver3d(
+        self,
+        x,
+        y,
+        z,
+        u,
+        v,
+        w,
+        color,
+        scale,
+        mode,
+        resolution=8,
+        glyph_height=None,
+        glyph_center=None,
+        glyph_resolution=None,
+        opacity=1.0,
+        scale_mode="none",
+        scalars=None,
+        backface_culling=False,
+        colormap=None,
+        vmin=None,
+        vmax=None,
+        line_width=2.0,
+        name=None,
+    ):
         """Add quiver3d in the scene.
 
         Parameters
@@ -295,7 +453,7 @@ class _AbstractRenderer(ABC):
             The optional scalar data to use.
         backface_culling : bool
             If True, enable backface culling on the quiver.
-        colormap :
+        colormap : str | np.ndarray | matplotlib.colors.Colormap | None
             The colormap to use.
         vmin : float | None
             vmin is used to scale the colormap.
@@ -316,7 +474,7 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def text2d(self, x_window, y_window, text, size=14, color='white'):
+    def text2d(self, x_window, y_window, text, size=14, color="white"):
         """Add 2d text in the scene.
 
         Parameters
@@ -339,7 +497,7 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def text3d(self, x, y, z, text, width, color='white'):
+    def text3d(self, x, y, z, text, width, color="white"):
         """Add 2d text in the scene.
 
         Parameters
@@ -362,21 +520,20 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def scalarbar(self, source, color="white", title=None, n_labels=4,
-                  bgcolor=None):
+    def scalarbar(self, source, color="white", title=None, n_labels=4, bgcolor=None):
         """Add a scalar bar in the scene.
 
         Parameters
         ----------
-        source :
+        source
             The object of the scene used for the colormap.
-        color :
+        color : tuple | str
             The color of the label text.
         title : str | None
             The title of the scalar bar.
         n_labels : int | None
             The number of labels to display on the scalar bar.
-        bgcolor :
+        bgcolor : tuple | str
             The color of the background when there is transparency.
         """
         pass
@@ -392,8 +549,14 @@ class _AbstractRenderer(ABC):
         pass
 
     @abstractclassmethod
-    def set_camera(self, azimuth=None, elevation=None, distance=None,
-                   focalpoint=None, roll=None, reset_camera=True):
+    def set_camera(
+        self,
+        azimuth=None,
+        elevation=None,
+        distance=None,
+        focalpoint=None,
+        roll=None,
+    ):
         """Configure the camera of the scene.
 
         Parameters
@@ -408,18 +571,11 @@ class _AbstractRenderer(ABC):
             The focal point of the camera: (x, y, z).
         roll : float
             The rotation of the camera along its axis.
-        reset_camera : bool
-           If True, reset the camera properties beforehand.
         """
         pass
 
     @abstractclassmethod
-    def reset_camera(self):
-        """Reset the camera properties."""
-        pass
-
-    @abstractclassmethod
-    def screenshot(self, mode='rgb', filename=None):
+    def screenshot(self, mode="rgb", filename=None):
         """Take a screenshot of the scene.
 
         Parameters
@@ -445,11 +601,6 @@ class _AbstractRenderer(ABC):
         """
         pass
 
-    @abstractclassmethod
-    def enable_depth_peeling(self):
-        """Enable depth peeling."""
-        pass
-
     @abstractclassmethod
     def remove_mesh(self, mesh_data):
         """Remove the given mesh from the scene.
@@ -462,18 +613,403 @@ class _AbstractRenderer(ABC):
         pass
 
 
-class _AbstractToolBar(ABC):
-    @abstractmethod
-    def _tool_bar_load_icons(self):
+# -------------------
+# Widget Abstractions
+# -------------------
+
+
+class _AbstractWidget(ABC):
+    @abstractclassmethod
+    def __init__(self):
         pass
 
+    @abstractmethod
+    def _show(self):
+        pass
+
+    @abstractmethod
+    def _hide(self):
+        pass
+
+    @abstractmethod
+    def _set_enabled(self, state):
+        pass
+
+    @abstractmethod
+    def _is_enabled(self):
+        pass
+
+    @abstractmethod
+    def _update(self, repaint=True):
+        pass
+
+    @abstractmethod
+    def _set_style(self, style):
+        pass
+
+    @abstractmethod
+    def _get_tooltip(self):
+        pass
+
+    @abstractmethod
+    def _set_tooltip(self, tooltip: str):
+        pass
+
+    @abstractmethod
+    def _add_keypress(self, callback):
+        pass
+
+    @abstractmethod
+    def _trigger_keypress(self, key):
+        pass
+
+    @abstractmethod
+    def _set_focus(self):
+        pass
+
+    @abstractmethod
+    def _set_layout(self, layout):
+        pass
+
+    @abstractmethod
+    def _set_theme(self, theme):
+        pass
+
+    @abstractmethod
+    def _set_size(self, width=None, height=None):
+        pass
+
+
+class _AbstractLabel(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, center=False, selectable=False):
+        pass
+
+
+class _AbstractText(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value=None, placeholder=None, callback=None):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+
+class _AbstractButton(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, callback, icon=None):
+        pass
+
+    @abstractmethod
+    def _click(self):
+        pass
+
+    @abstractmethod
+    def _set_icon(self, icon):
+        pass
+
+
+class _AbstractSlider(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, rng, callback, horizontal=True):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+    @abstractmethod
+    def _set_range(self, rng):
+        pass
+
+
+class _AbstractProgressBar(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, count):
+        pass
+
+    @abstractmethod
+    def _increment(self):
+        pass
+
+
+class _AbstractCheckBox(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, callback):
+        pass
+
+    @abstractmethod
+    def _set_checked(self, checked):
+        pass
+
+    @abstractmethod
+    def _get_checked(self):
+        pass
+
+
+class _AbstractSpinBox(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, rng, callback, step=None):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractComboBox(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, items, callback):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractRadioButtons(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, items, callback):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractGroupBox(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, name, items):
+        pass
+
+
+class _AbstractFileButton(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(
+        self,
+        callback,
+        content_filter=None,
+        initial_directory=None,
+        save=False,
+        is_directory=False,
+        icon="folder",
+        window=None,
+    ):
+        pass
+
+
+class _AbstractPlayMenu(_AbstractWidget):
+    @abstractclassmethod
+    def __init__(self, value, rng, callback):
+        pass
+
+    @abstractmethod
+    def _play(self):
+        pass
+
+    @abstractmethod
+    def _pause(self):
+        pass
+
+    @abstractmethod
+    def _reset(self):
+        pass
+
+    @abstractmethod
+    def _loop(self):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+
+class _AbstractPopup(_AbstractWidget):
+    _supported_button_names = ["Ok"]
+    # TODO: Add back support for below, file browser takes care of most
+    # so no big need currently
+    """
+    # from QMessageBox.StandardButtons
+    ['Ok', 'Open', 'Save', 'Cancel', 'Close', 'Discard', 'Apply',
+    'Reset', 'RestoreDefaults', 'Help', 'SaveAll', 'Yes',
+    'YesToAll', 'No', 'NoToAll', 'Abort', 'Retry', 'Ignore']
+    """
+
+    _supported_icon_names = ["question", "information", "warning", "critical"]
+
+    @abstractmethod
+    def __init__(
+        self,
+        title,
+        text,
+        info_text=None,
+        callback=None,
+        icon="Warning",
+        buttons=None,
+        window=None,
+    ):
+        pass
+
+    @abstractmethod
+    def _click(self, value):
+        pass
+
+
+# -------
+# Layouts
+# -------
+
+
+class _AbstractBoxLayout(ABC):
+    @abstractmethod
+    def _add_widget(self, widget):
+        pass
+
+    @abstractmethod
+    def _add_stretch(self, amount=1):
+        pass
+
+
+class _AbstractHBoxLayout(_AbstractBoxLayout):
+    @abstractmethod
+    def __init__(self, height=None, scroll=None):
+        pass
+
+
+class _AbstractVBoxLayout(_AbstractBoxLayout):
+    @abstractmethod
+    def __init__(self, width=None, scroll=None):
+        pass
+
+
+class _AbstractGridLayout(ABC):
+    @abstractmethod
+    def __init__(self, height=None, width=None, scroll=None):
+        pass
+
+    @abstractmethod
+    def _add_widget(self, widget, row=None, col=None):
+        pass
+
+
+class _AbstractAppWindow(ABC):
+    def __init__(self, size=None, fullscreen=False):
+        pass
+
+    @abstractmethod
+    def _set_central_layout(self, central_layout):
+        pass
+
+    @abstractmethod
+    def _get_dpi(self):
+        pass
+
+    @abstractmethod
+    def _get_size(self):
+        pass
+
+    @abstractmethod
+    def _get_cursor(self):
+        pass
+
+    @abstractmethod
+    def _set_cursor(self, cursor):
+        pass
+
+    @abstractmethod
+    def _new_cursor(self, name):
+        pass
+
+    @abstractmethod
+    def _close_connect(self, func, *, after=True):
+        pass
+
+    @abstractmethod
+    def _close_disconnect(self, after=True):
+        pass
+
+    @abstractmethod
+    def _clean(self):
+        pass
+
+    @abstractmethod
+    def _show(self, block=False):
+        pass
+
+    @abstractmethod
+    def _close(self):
+        pass
+
+
+# -------------------
+# Matplotlib Canvases
+# -------------------
+
+
+class _AbstractCanvas(ABC):
+    def __init__(self, width=None, height=None, dpi=None):
+        """Initialize the matplotlib Canvas."""
+        pass
+
+    def show(self):
+        """Show the canvas."""
+        if self.manager is None:
+            self.show()
+        else:
+            self.manager.show()
+
+    def close(self):
+        """Close the canvas."""
+        self.close()
+
+    def update(self):
+        """Update the canvas."""
+        self.fig.canvas.draw()
+        self.fig.canvas.flush_events()
+
+    def clear(self):
+        """Clear internal variables."""
+        self.close()
+        self.ax.clear()
+        self.fig.clear()
+        self.manager = None
+
+    @abstractmethod
+    def _set_size(self, width=None, height=None):
+        pass
+
+
+# ------------------------------------
+# Non-object-based Widget Abstractions
+# ------------------------------------
+# These are planned to be removed in favor of the simpler, object-
+# oriented abstractions above when time allows.
+
+
+class _AbstractToolBar(ABC):
     @abstractmethod
     def _tool_bar_initialize(self, name="default", window=None):
         pass
 
     @abstractmethod
-    def _tool_bar_add_button(self, name, desc, func, icon_name=None,
-                             shortcut=None):
+    def _tool_bar_add_button(self, name, desc, func, *, icon_name=None, shortcut=None):
         pass
 
     @abstractmethod
@@ -489,22 +1025,19 @@ class _AbstractToolBar(ABC):
         pass
 
     @abstractmethod
-    def _tool_bar_add_file_button(self, name, desc, func, shortcut=None):
+    def _tool_bar_add_file_button(self, name, desc, func, *, shortcut=None):
         pass
 
     @abstractmethod
-    def _tool_bar_add_play_button(self, name, desc, func, shortcut=None):
-        pass
-
-    @abstractmethod
-    def _tool_bar_set_theme(self, theme):
+    def _tool_bar_add_play_button(self, name, desc, func, *, shortcut=None):
         pass
 
 
 class _AbstractDock(ABC):
     @abstractmethod
-    def _dock_initialize(self, window=None, name="Controls",
-                         area="left"):
+    def _dock_initialize(
+        self, window=None, name="Controls", area="left", max_width=None
+    ):
         pass
 
     @abstractmethod
@@ -528,54 +1061,96 @@ class _AbstractDock(ABC):
         pass
 
     @abstractmethod
-    def _dock_add_label(self, value, align=False, layout=None):
+    def _dock_add_label(self, value, *, align=False, layout=None, selectable=False):
         pass
 
     @abstractmethod
-    def _dock_add_button(self, name, callback, layout=None):
+    def _dock_add_button(
+        self,
+        name,
+        callback,
+        *,
+        style="pushbutton",
+        icon=None,
+        tooltip=None,
+        layout=None,
+    ):
         pass
 
     @abstractmethod
-    def _dock_named_layout(self, name, layout=None, compact=True):
+    def _dock_named_layout(self, name, *, layout=None, compact=True):
         pass
 
     @abstractmethod
-    def _dock_add_slider(self, name, value, rng, callback,
-                         compact=True, double=False, layout=None):
+    def _dock_add_slider(
+        self,
+        name,
+        value,
+        rng,
+        callback,
+        *,
+        compact=True,
+        double=False,
+        tooltip=None,
+        layout=None,
+    ):
         pass
 
     @abstractmethod
-    def _dock_add_check_box(self, name, value, callback, layout=None):
+    def _dock_add_check_box(self, name, value, callback, *, tooltip=None, layout=None):
         pass
 
     @abstractmethod
-    def _dock_add_spin_box(self, name, value, rng, callback,
-                           compact=True, double=True, step=None,
-                           layout=None):
+    def _dock_add_spin_box(
+        self,
+        name,
+        value,
+        rng,
+        callback,
+        *,
+        compact=True,
+        double=True,
+        step=None,
+        tooltip=None,
+        layout=None,
+    ):
         pass
 
     @abstractmethod
-    def _dock_add_combo_box(self, name, value, rng,
-                            callback, compact=True, layout=None):
+    def _dock_add_combo_box(
+        self, name, value, rng, callback, *, compact=True, tooltip=None, layout=None
+    ):
         pass
 
     @abstractmethod
-    def _dock_add_radio_buttons(self, value, rng, callback, vertical=True,
-                                layout=None):
+    def _dock_add_radio_buttons(
+        self, value, rng, callback, *, vertical=True, layout=None
+    ):
         pass
 
     @abstractmethod
-    def _dock_add_group_box(self, name, layout=None):
+    def _dock_add_group_box(self, name, *, collapse=None, layout=None):
         pass
 
     @abstractmethod
-    def _dock_add_text(self, name, value, placeholder, layout=None):
+    def _dock_add_text(self, name, value, placeholder, *, callback=None, layout=None):
         pass
 
     @abstractmethod
-    def _dock_add_file_button(self, name, desc, func, value=None, save=False,
-                              directory=False, input_text_widget=True,
-                              placeholder="Type a file name", layout=None):
+    def _dock_add_file_button(
+        self,
+        name,
+        desc,
+        func,
+        *,
+        filter_=None,
+        initial_directory=None,
+        save=False,
+        is_directory=False,
+        icon=False,
+        tooltip=None,
+        layout=None,
+    ):
         pass
 
 
@@ -599,7 +1174,7 @@ class _AbstractStatusBar(ABC):
         pass
 
     @abstractmethod
-    def _status_bar_add_label(self, value, stretch=0):
+    def _status_bar_add_label(self, value, *, stretch=0):
         pass
 
     @abstractmethod
@@ -613,8 +1188,38 @@ class _AbstractStatusBar(ABC):
 
 class _AbstractPlayback(ABC):
     @abstractmethod
-    def _playback_initialize(self, func, timeout, value, rng,
-                             time_widget, play_widget):
+    def _playback_initialize(self, func, timeout, value, rng, time_widget, play_widget):
+        pass
+
+
+class _AbstractKeyPress(ABC):
+    @abstractmethod
+    def _keypress_initialize(self, widget=None):
+        pass
+
+    @abstractmethod
+    def _keypress_add(self, shortcut, callback):
+        pass
+
+    @abstractmethod
+    def _keypress_trigger(self, shortcut):
+        pass
+
+
+class _AbstractDialog(ABC):
+    @abstractmethod
+    def _dialog_create(
+        self,
+        title,
+        text,
+        info_text,
+        callback,
+        *,
+        icon="Warning",
+        buttons=(),
+        modal=True,
+        window=None,
+    ):
         pass
 
 
@@ -624,7 +1229,11 @@ class _AbstractLayout(ABC):
         pass
 
     @abstractmethod
-    def _layout_add_widget(self, layout, widget, stretch=0):
+    def _layout_add_widget(self, layout, widget, stretch=0, *, row=None, col=None):
+        pass
+
+    @abstractmethod
+    def _layout_create(self, orientation="vertical"):
         pass
 
 
@@ -642,7 +1251,7 @@ class _AbstractWidgetList(ABC):
         pass
 
 
-class _AbstractWidget(ABC):
+class _AbstractWdgt(ABC):
     def __init__(self, widget):
         self._widget = widget
 
@@ -674,10 +1283,43 @@ class _AbstractWidget(ABC):
     def set_enabled(self, state):
         pass
 
+    @abstractmethod
+    def is_enabled(self):
+        pass
+
     @abstractmethod
     def update(self, repaint=True):
         pass
 
+    @abstractmethod
+    def get_tooltip(self):
+        pass
+
+    @abstractmethod
+    def set_tooltip(self, tooltip: str):
+        pass
+
+    @abstractmethod
+    def set_style(self, style):
+        pass
+
+
+class _AbstractAction(ABC):
+    def __init__(self, action):
+        self._action = action
+
+    @abstractmethod
+    def trigger(self):
+        pass
+
+    @abstractmethod
+    def set_icon(self):
+        pass
+
+    @abstractmethod
+    def set_shortcut(self):
+        pass
+
 
 class _AbstractMplInterface(ABC):
     @abstractmethod
@@ -688,31 +1330,21 @@ class _AbstractMplInterface(ABC):
 class _AbstractMplCanvas(ABC):
     def __init__(self, width, height, dpi):
         """Initialize the MplCanvas."""
-        from matplotlib import rc_context
         from matplotlib.figure import Figure
-        # prefer constrained layout here but live with tight_layout otherwise
-        context = nullcontext
-        self._extra_events = ('resize',)
-        try:
-            context = rc_context({'figure.constrained_layout.use': True})
-            self._extra_events = ()
-        except KeyError:
-            pass
-        with context:
-            self.fig = Figure(figsize=(width, height), dpi=dpi)
+
+        self._extra_events = ("resize",)
+        self.fig = Figure(figsize=(width, height), dpi=dpi, layout="constrained")
         self.axes = self.fig.add_subplot(111)
-        self.axes.set(xlabel='Time (sec)', ylabel='Activation (AU)')
+        self.axes.set(xlabel="Time (s)", ylabel="Activation (AU)")
         self.manager = None
 
     def _connect(self):
-        for event in ('button_press', 'motion_notify') + self._extra_events:
-            self.canvas.mpl_connect(
-                event + '_event', getattr(self, 'on_' + event))
+        for event in ("button_press", "motion_notify") + self._extra_events:
+            self.canvas.mpl_connect(event + "_event", getattr(self, "on_" + event))
 
     def plot(self, x, y, label, update=True, **kwargs):
         """Plot a curve."""
-        line, = self.axes.plot(
-            x, y, label=label, **kwargs)
+        (line,) = self.axes.plot(x, y, label=label, **kwargs)
         if update:
             self.update_plot()
         return line
@@ -727,7 +1359,7 @@ class _AbstractMplCanvas(ABC):
     def update_plot(self):
         """Update the plot."""
         with warnings.catch_warnings(record=True):
-            warnings.filterwarnings('ignore', 'constrained_layout')
+            warnings.filterwarnings("ignore", "constrained_layout")
             self.canvas.draw()
 
     def set_color(self, bg_color, fg_color):
@@ -735,12 +1367,12 @@ class _AbstractMplCanvas(ABC):
         self.axes.set_facecolor(bg_color)
         self.axes.xaxis.label.set_color(fg_color)
         self.axes.yaxis.label.set_color(fg_color)
-        self.axes.spines['top'].set_color(fg_color)
-        self.axes.spines['bottom'].set_color(fg_color)
-        self.axes.spines['left'].set_color(fg_color)
-        self.axes.spines['right'].set_color(fg_color)
-        self.axes.tick_params(axis='x', colors=fg_color)
-        self.axes.tick_params(axis='y', colors=fg_color)
+        self.axes.spines["top"].set_color(fg_color)
+        self.axes.spines["bottom"].set_color(fg_color)
+        self.axes.spines["left"].set_color(fg_color)
+        self.axes.spines["right"].set_color(fg_color)
+        self.axes.tick_params(axis="x", colors=fg_color)
+        self.axes.tick_params(axis="y", colors=fg_color)
         self.fig.patch.set_facecolor(bg_color)
 
     def show(self):
@@ -764,7 +1396,7 @@ class _AbstractMplCanvas(ABC):
 
     def on_resize(self, event):
         """Handle resize events."""
-        tight_layout(fig=self.axes.figure)
+        pass
 
 
 class _AbstractBrainMplCanvas(_AbstractMplCanvas):
@@ -772,14 +1404,15 @@ class _AbstractBrainMplCanvas(_AbstractMplCanvas):
         """Initialize the MplCanvas."""
         super().__init__(width, height, dpi)
         self.brain = brain
-        self.time_func = brain.callbacks["time"]
 
     def update_plot(self):
         """Update the plot."""
         leg = self.axes.legend(
-            prop={'family': 'monospace', 'size': 'small'},
-            framealpha=0.5, handlelength=1.,
-            facecolor=self.brain._bg_color)
+            prop={"family": "monospace", "size": "small"},
+            framealpha=0.5,
+            handlelength=1.0,
+            facecolor=self.brain._bg_color,
+        )
         for text in leg.get_texts():
             text.set_color(self.brain._fg_color)
         super().update_plot()
@@ -787,11 +1420,9 @@ class _AbstractBrainMplCanvas(_AbstractMplCanvas):
     def on_button_press(self, event):
         """Handle button presses."""
         # left click (and maybe drag) in progress in axes
-        if (event.inaxes != self.axes or
-                event.button != 1):
+        if event.inaxes != self.axes or event.button != 1:
             return
-        self.time_func(
-            event.xdata, update_widget=True, time_as_index=False)
+        publish(self.brain, TimeChange(time=event.xdata))
 
     on_motion_notify = on_button_press  # for now they can be the same
 
@@ -802,7 +1433,8 @@ class _AbstractBrainMplCanvas(_AbstractMplCanvas):
 
 
 class _AbstractWindow(ABC):
-    def _window_initialize(self):
+    def _window_initialize(self, *, window=None, central_layout=None, fullscreen=False):
+        self._icons = dict()
         self._window = None
         self._interactor = None
         self._mplcanvas = None
@@ -811,7 +1443,15 @@ class _AbstractWindow(ABC):
         self._interactor_fraction = None
 
     @abstractmethod
-    def _window_close_connect(self, func):
+    def _window_load_icons(self):
+        pass
+
+    @abstractmethod
+    def _window_close_connect(self, func, *, after=True):
+        pass
+
+    @abstractmethod
+    def _window_close_disconnect(self, after=True):
         pass
 
     @abstractmethod
@@ -834,8 +1474,9 @@ class _AbstractWindow(ABC):
         pass
 
     @abstractmethod
-    def _window_get_mplcanvas(self, brain, interactor_fraction, show_traces,
-                              separate_canvas):
+    def _window_get_mplcanvas(
+        self, brain, interactor_fraction, show_traces, separate_canvas
+    ):
         pass
 
     @abstractmethod
@@ -861,3 +1502,7 @@ class _AbstractWindow(ABC):
     @abstractmethod
     def _window_set_theme(self, theme):
         pass
+
+    @abstractmethod
+    def _window_create(self):
+        pass
